from DateTime import DateTime
from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_CONTENT_TESTING
from ftw.simplelayout.utils import IS_PLONE_5
from ftw.testbrowser import browsing
from plone import api
from plone.registry.interfaces import IRegistry
from Products.CMFPlone.utils import parent
from unittest import TestCase
from zope.component import queryUtility
import os
import transaction


if IS_PLONE_5:
    from Products.CMFPlone.interfaces.controlpanel import IImagingSchema


def get_allowed_size_names():
    if IS_PLONE_5:
        registry = queryUtility(IRegistry)
        sizes = registry.forInterface(IImagingSchema, prefix="plone", check=False).allowed_sizes

    else:
        ptool = api.portal.get_tool('portal_properties')
        sizes = ptool.get('imaging_properties').allowed_sizes

    return map(lambda size: size.split(' ')[0], sizes)


class TestGalleryBlock(TestCase):

    layer = FTW_SIMPLELAYOUT_CONTENT_TESTING

    def setUp(self):
        super(TestGalleryBlock, self).setUp()
        self.portal = self.layer['portal']
        self.page = create(Builder('sl content page').titled(u'A page'))

    def test_scale_is_available(self):
        name = 'simplelayout_galleryblock'
        names = get_allowed_size_names()
        self.assertIn(name, names,
                      "The simplelayout_galleryblock scale is not available." +
                      "Available scales: {0}".format(names))

    @browsing
    def test_get_images_only_returns_images_of_current_context(self, browser):
        gallery1 = create(Builder('sl galleryblock')
                          .titled('Galleryblock 1')
                          .having(show_title=True)
                          .within(self.page))

        create(Builder('image')
               .titled('gallery1_img')
               .with_dummy_content()
               .within(gallery1))

        gallery2 = create(Builder('sl galleryblock')
                          .titled('Galleryblock 2')
                          .having(show_title=True)
                          .within(self.page))

        create(Builder('image')
               .titled('gallery2_img')
               .with_dummy_content()
               .within(gallery2))

        browser.login().visit(self.page)

        css_items = browser.css(
            'a.colorboxLink[rel="colorbox-galleryblock-1"] img')
        nodes = [node.get('alt').split(',')[0] for node in css_items]

        self.assertEqual(nodes, ['gallery1_img'])

    @browsing
    def test_rendering(self, browser):
        create(Builder('sl galleryblock')
               .titled('My galleryblock')
               .having(show_title=True)
               .within(self.page))

        browser.login().visit(self.page)

        self.assertTrue(browser.css('.sl-block'))

        self.assertEquals(u'My galleryblock',
                          browser.css('.sl-block h2').first.text)

    @browsing
    def test_all_the_images_will_be_displayed(self, browser):
        block = create(Builder('sl galleryblock')
                       .titled('My galleryblock')
                       .having(show_title=True)
                       .within(self.page))

        create(Builder('image')
               .titled('Test image')
               .with_dummy_content()
               .within(block))

        create(Builder('image')
               .titled('Test im\xc3\xa4ge')
               .with_dummy_content()
               .within(block))

        browser.login().visit(self.page)
        self.assertEquals(2, len(browser.css('.sl-block-content .galleryblockImageWrapper img')))

    @browsing
    def test_each_gallery_has_a_unique_rel_name(self, browser):
        gallerie_1 = create(Builder('sl galleryblock')
                            .titled('My galleryblock')
                            .having(show_title=True)
                            .within(self.page))

        create(Builder('image')
               .titled('Test image')
               .with_dummy_content()
               .within(gallerie_1))

        create(Builder('image')
               .titled('Test image')
               .with_dummy_content()
               .within(gallerie_1))

        gallerie_2 = create(Builder('sl galleryblock')
                            .titled('My galleryblock')
                            .having(show_title=True)
                            .within(self.page))

        create(Builder('image')
               .titled('Test image')
               .with_dummy_content()
               .within(gallerie_2))

        create(Builder('image')
               .titled('Test image')
               .with_dummy_content()
               .within(gallerie_2))

        browser.login().visit(self.page)

        # Should be two images in each gallery.
        # The rel is 'colorbox-{block-id}'

        self.assertEquals(
            2,
            len(browser.css('.sl-block-content a[rel="colorbox-{0}"]'.format(
                gallerie_1.getId()))))

        self.assertEquals(
            2,
            len(browser.css('.sl-block-content a[rel="colorbox-{0}"]'.format(
                gallerie_2.getId()))))

    @browsing
    def test_galleryblock_title_not_rendered_when_empty(self, browser):
        """
        This test makes sure that the title of the block is only rendered
        if there is a title. Otherwise we'll end up with an empty HTML
        tag in the template.
        """
        galleryblock = create(Builder('sl galleryblock')
                              .titled('My galleryblock')
                              .having(show_title=True)
                              .within(self.page))

        browser.login().visit(self.page)

        title_css_selector = '.ftw-simplelayout-galleryblock h2'

        # Make sure the title is here (in its tag).
        self.assertEqual('My galleryblock',
                         browser.css(title_css_selector).first.text)

        # Remove the title of the block and make sure the tag is no longer
        # there.
        galleryblock.title = ''
        transaction.commit()
        browser.login().visit(self.page)
        self.assertEqual([], browser.css(title_css_selector))

    @browsing
    def test_hidden_galleryblock_has_special_class(self, browser):
        """
        This test makes sure that a special class is available on the block
        if the block is hidden. This can be used to visually highlight
        hidden blocks.
        """
        galleryblock = create(Builder('sl galleryblock')
                              .titled('My galleryblock')
                              .having(show_title=True)
                              .having(is_hidden=True)
                              .within(self.page))

        browser.login()

        # The block must have a class "hidden".
        browser.visit(self.page)
        self.assertEqual(
            'sl-block ftw-simplelayout-galleryblock hidden',
            browser.css('.ftw-simplelayout-galleryblock').first.attrib['class']
        )

        # Edit the block and make it appear again.
        browser.visit(galleryblock, view='edit')
        browser.fill({'Hide the block': False}).submit()

        # The block must no longer have a class "hidden".
        browser.visit(self.page)
        self.assertEqual(
            'sl-block ftw-simplelayout-galleryblock',
            browser.css('.ftw-simplelayout-galleryblock').first.attrib['class']
        )

    @browsing
    def test_hidden_galleryblock_not_visible_without_edit_permission(self,
                                                                     browser):
        """
        This test makes sure that users without edit permission, e.g. the
        anonymous user, do not see the hidden block.
        """
        create(Builder('sl galleryblock')
               .titled('My galleryblock')
               .having(show_title=True)
               .having(is_hidden=True)
               .within(self.page))

        # Make sure an anonymous user cannot see the block.
        browser.logout().visit(self.page)
        self.assertEqual(
            [],
            browser.css('.ftw-simplelayout-galleryblock')
        )

        # Login to make sure the block is visible for admin users.
        browser.login().visit(self.page)
        self.assertEqual(
            ['My galleryblock'],
            browser.css('.ftw-simplelayout-galleryblock h2').text
        )

    @browsing
    def test_available_sort_options(self, browser):
        gallery_block = create(Builder('sl galleryblock')
                               .titled('My galleryblock')
                               .within(self.page))
        browser.login()
        browser.visit(gallery_block, view='edit')
        self.assertEqual(
            [
                ('sortable_title', 'Title'),
                ('modified', 'Modification date'),
                ('id', 'ID'),
                ('getObjPositionInParent', 'Position in Folder'),
            ],
            browser.forms['form'].find_field('Sort by').options
        )

    @browsing
    def test_sort_by_title_ascending(self, browser):
        gallery = create(Builder('sl galleryblock')
                         .titled('My galleryblock')
                         .having(sort_on='sortable_title')
                         .having(sort_order='ascending')
                         .within(self.page))

        create(Builder('image')
               .titled('a_img')
               .with_dummy_content()
               .within(gallery))

        create(Builder('image')
               .titled('b_img')
               .with_dummy_content()
               .within(gallery))

        create(Builder('image')
               .titled('c_img')
               .with_dummy_content()
               .within(gallery))

        browser.login().visit(self.page)

        img_list = browser.css('.colorboxLink img')
        img_title_list = []

        for img in img_list:
            img_title_list.append(img.get('alt').split(',')[0])

        self.assertEqual(img_title_list, ['a_img', 'b_img', 'c_img'])

    @browsing
    def test_sort_by_title_descending(self, browser):
        gallery = create(Builder('sl galleryblock')
                         .titled('My galleryblock')
                         .having(sort_on='sortable_title')
                         .having(sort_order='descending')
                         .within(self.page))

        create(Builder('image')
               .titled('a_img')
               .with_dummy_content()
               .within(gallery))

        create(Builder('image')
               .titled('b_img')
               .with_dummy_content()
               .within(gallery))

        create(Builder('image')
               .titled('c_img')
               .with_dummy_content()
               .within(gallery))

        browser.login().visit(self.page)

        img_list = browser.css('.colorboxLink img')
        img_title_list = []

        for img in img_list:
            img_title_list.append(img.get('alt').split(',')[0])

        self.assertEqual(img_title_list, ['c_img', 'b_img', 'a_img'])

    @browsing
    def test_sort_by_modified_ascending(self, browser):
        gallery = create(Builder('sl galleryblock')
                         .titled('My galleryblock')
                         .having(sort_on='modified')
                         .having(sort_order='ascending')
                         .within(self.page))

        create(Builder('image')
               .titled('a_img')
               .with_dummy_content()
               .within(gallery)
               .with_modification_date(DateTime("2014-01-01")))

        create(Builder('image')
               .titled('b_img')
               .with_dummy_content()
               .within(gallery)
               .with_modification_date(DateTime("2016-01-01")))

        create(Builder('image')
               .titled('c_img')
               .with_dummy_content()
               .within(gallery)
               .with_modification_date(DateTime("2015-01-01")))

        browser.login().visit(self.page)

        img_list = browser.css('.colorboxLink img')
        img_title_list = []

        for img in img_list:
            img_title_list.append(img.get('alt').split(',')[0])

        self.assertEqual(img_title_list, ['a_img', 'c_img', 'b_img'])

    @browsing
    def test_sort_by_modified_descending(self, browser):
        gallery = create(Builder('sl galleryblock')
                         .titled('My galleryblock')
                         .having(sort_on='modified')
                         .having(sort_order='descending')
                         .within(self.page))

        create(Builder('image')
               .titled('a_img')
               .with_dummy_content()
               .within(gallery)
               .with_modification_date(DateTime("2014-01-01")))

        create(Builder('image')
               .titled('b_img')
               .with_dummy_content()
               .within(gallery)
               .with_modification_date(DateTime("2016-01-01")))

        create(Builder('image')
               .titled('c_img')
               .with_dummy_content()
               .within(gallery)
               .with_modification_date(DateTime("2015-01-01")))

        browser.login().visit(self.page)

        img_list = browser.css('.colorboxLink img')
        img_title_list = []

        for img in img_list:
            img_title_list.append(img.get('alt').split(',')[0])

        self.assertEqual(img_title_list, ['b_img', 'c_img', 'a_img'])

    @browsing
    def test_sort_by_position_in_parent_ascending(self, browser):
        gallery_block = create(Builder('sl galleryblock')
                               .titled('My galleryblock')
                               .having(sort_on='getObjPositionInParent')
                               .having(sort_order='ascending')
                               .within(self.page))

        create(Builder('image')
               .titled('a_img')
               .with_dummy_content()
               .within(gallery_block))

        img_b = create(Builder('image')
                       .titled('b_img')
                       .with_dummy_content()
                       .within(gallery_block))

        create(Builder('image')
               .titled('c_img')
               .with_dummy_content()
               .within(gallery_block))

        # Move "img_b" to bottom.
        parent(img_b).moveObjectsToBottom(img_b.getId())
        transaction.commit()

        browser.login()
        browser.visit(gallery_block, view='block_view')
        self.assertEqual(
            ['a_img', 'c_img', 'b_img'],
            [node.attrib['title'] for node in browser.css('a')]
        )

    @browsing
    def test_sort_by_position_in_parent_descending(self, browser):
        gallery_block = create(Builder('sl galleryblock')
                               .titled('My galleryblock')
                               .having(sort_on='getObjPositionInParent')
                               .having(sort_order='descending')
                               .within(self.page))

        create(Builder('image')
               .titled('a_img')
               .with_dummy_content()
               .within(gallery_block))

        img_b = create(Builder('image')
                       .titled('b_img')
                       .with_dummy_content()
                       .within(gallery_block))

        create(Builder('image')
               .titled('c_img')
               .with_dummy_content()
               .within(gallery_block))

        # Move "img_b" to bottom.
        parent(img_b).moveObjectsToBottom(img_b.getId())
        transaction.commit()

        browser.login()
        browser.visit(gallery_block, view='block_view')
        self.assertEqual(
            ['b_img', 'c_img', 'a_img'],
            [node.attrib['title'] for node in browser.css('a')]
        )

    @browsing
    def test_sort_by_id_ascending(self, browser):
        gallery_block = create(Builder('sl galleryblock')
                               .titled('My galleryblock')
                               .having(sort_on='id')
                               .having(sort_order='ascending')
                               .within(self.page))

        create(Builder('image')
               .titled('c_img')
               .with_dummy_content()
               .within(gallery_block))

        create(Builder('image')
               .titled('a_img')
               .with_dummy_content()
               .within(gallery_block))

        create(Builder('image')
               .titled('b_img')
               .with_dummy_content()
               .within(gallery_block))

        browser.login()
        browser.visit(gallery_block, view='block_view')
        self.assertEqual(
            ['a_img', 'b_img', 'c_img'],
            [node.attrib['title'] for node in browser.css('a')]
        )

    @browsing
    def test_sort_by_id_descending(self, browser):
        gallery_block = create(Builder('sl galleryblock')
                               .titled('My galleryblock')
                               .having(sort_on='id')
                               .having(sort_order='descending')
                               .within(self.page))

        create(Builder('image')
               .titled('c_img')
               .with_dummy_content()
               .within(gallery_block))

        create(Builder('image')
               .titled('a_img')
               .with_dummy_content()
               .within(gallery_block))

        create(Builder('image')
               .titled('b_img')
               .with_dummy_content()
               .within(gallery_block))

        browser.login()
        browser.visit(gallery_block, view='block_view')
        self.assertEqual(
            ['c_img', 'b_img', 'a_img'],
            [node.attrib['title'] for node in browser.css('a')]
        )

    @browsing
    def test_image_title_with_umlaut(self, browser):
        """
        This test makes sure that gallery block does not fail when
        it contains image objects having umlauts in their title.
        """
        gallery = create(Builder('sl galleryblock')
                         .titled('My Gallery Block')
                         .within(self.page))
        create(Builder('image')
               .titled(u'T\xe4st')
               .with_dummy_content()
               .within(gallery))

        browser.login()
        browser.open(self.page)
        self.assertEqual(
            u'T\xe4st',
            browser.css(
                '.ftw-simplelayout-galleryblock .sl-block-content a').first.attrib['title']
        )
        self.assertEqual(
            u'T\xe4st, enlarged picture.',
            browser.css(
                '.ftw-simplelayout-galleryblock .sl-block-content a img').first.attrib['alt']
        )

    @browsing
    def test_render_fallback_image_for_truncated_images(self, browser):
        block = create(Builder('sl galleryblock')
                       .titled('My galleryblock')
                       .having(show_title=True)
                       .within(self.page))

        # Create an image inside the gallery block using a truncated image
        # file.
        current_folder = os.path.dirname(__file__)
        truncated_image_path = os.path.join(
            current_folder, 'assets', 'truncated_image.jpg')
        with open(truncated_image_path, 'rb') as truncated_image:
            create(Builder('image')
                   .titled('Test image')
                   .attach_file_containing(truncated_image.read())
                   .within(block))

        # The galleryblock renders a fallback image.
        browser.login().visit(self.page)
        self.assertEqual(
            'http://nohost/plone/++resource++ftw.simplelayout/image_unavailable.png',
            browser.css('.sl-block-content .galleryblockImageWrapper img').first.attrib['src']
        )



from Acquisition._Acquisition import aq_inner
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ftw.simplelayout.browser.blocks.base import BaseBlock
from ftw.simplelayout.contenttypes.contents import interfaces
from ftw.table.interfaces import ITableGenerator
from plone.dexterity.utils import safe_utf8
from zope.component import queryMultiAdapter
from zope.component import queryUtility


class FileListingBlockView(BaseBlock):
    """ListingBlock default view"""

    template = ViewPageTemplateFile('templates/listingblock.pt')
    table_template = ViewPageTemplateFile(
        'templates/ftw.table.custom.template.pt')

    def get_table_contents(self):
        catalog = getToolByName(self.context, 'portal_catalog')
        return catalog(self._build_query)

    @property
    def _build_query(self):
        query = {}
        path = '/'.join(self.context.getPhysicalPath())
        query['path'] = {'query': path, 'depth': 1}
        query['sort_on'] = self.context.sort_on
        query['sort_order'] = safe_utf8(self.context.sort_order)
        return query

    def _get_columns(self, column_id):
        adapter = queryMultiAdapter((self.context, self.request),
                                    interfaces.IListingBlockColumns)
        for column in adapter.columns():
            if column_id == column['column']:
                return column
        return None

    def _filtered_columns(self):
        for column_id in self.context.columns:
            column = self._get_columns(column_id)
            if column:
                yield column

    def render_table(self):
        # Use a custom table template, because we don't want a table header id.
        # The id value is moved to a css klass.
        # Reason: It's no allowed to have an id more than once (In case we
        # have more than one Listingblock on one contentpage)
        generator = queryUtility(ITableGenerator, 'ftw.tablegenerator')
        return generator.generate(
            self.get_table_contents(),
            list(self._filtered_columns()),
            sortable=False,
            css_mapping={'table': 'listing nosort'},
            template=self.table_template,
            options={'table_summary': self.context.Title()},
            selected=(self._build_query['sort_on'],
                      self._build_query['sort_order']))

    @property
    def can_add(self):
        context = aq_inner(self.context)
        mtool = getToolByName(context, 'portal_membership')
        permission = mtool.checkPermission(
            'ftw.simplelayout: Add FileListingBlock', context)
        return bool(permission)



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_CONTENT_TESTING
from ftw.testbrowser import browsing
from unittest import TestCase


class TestDefaultPageForm(TestCase):

    layer = FTW_SIMPLELAYOUT_CONTENT_TESTING

    @browsing
    def test_default_page_does_not_list_textblock(self, browser):
        page = create(Builder('sl content page'))
        create(Builder('sl textblock').within(page))
        create(Builder('sl content page').titled(u'subpage').within(page))

        browser.login()
        browser.visit(page, view='select_default_page')

        self.assertEquals(['subpage'], browser.css('#content-core label').text)



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING
from ftw.simplelayout.testing import SimplelayoutTestCase
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import statusmessages


class TestBlockStatusmessage(SimplelayoutTestCase):

    layer = FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING

    def setUp(self):
        self.setup_sample_ftis(self.layer['portal'])
        self.setup_block_views()
        self.page = create(Builder('sample container'))

    @browsing
    def test_NO_statusmessage_on_block_creation(self, browser):
        browser.login().visit(self.page, view="++add_block++SampleBlock")
        response = browser.json

        browser.parse(response['content'])
        browser.fill({'Title': u'This is a TextBlock',
                      'Text': u'Some text'})
        browser.find_button_by_label('Save').click()

        browser.visit(self.page)
        statusmessages.assert_no_messages()

    @browsing
    def test_NO_statusmessage_on_block_modification(self, browser):
        block = create(Builder('sample block'))
        browser.login().visit(block, view='edit.json')
        response = browser.json

        browser.parse(response['content'])
        browser.fill({'Title': u'This is a TextBlock',
                      'Text': u'Some text'})
        browser.find_button_by_label('Save').click()

        browser.visit(self.page)
        statusmessages.assert_no_messages()



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.properties import BLOCK_PROPERTIES_KEY
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_CONTENT_TESTING
from ftw.simplelayout.utils import get_block_html
from ftw.testbrowser import browsing
from Persistence import PersistentMapping
from unittest import TestCase
from zope.annotation import IAnnotations


class TestUtils(TestCase):

    layer = FTW_SIMPLELAYOUT_CONTENT_TESTING

    @browsing
    def test_get_html_block_with_unicode_viewname(self, browser):
        page = create(Builder('sl content page'))
        block = create(Builder('sl textblock')
                       .within(page)
                       .titled(u'My Text Block'))

        # Set a unicode view name.
        annotations = IAnnotations(block)
        if BLOCK_PROPERTIES_KEY not in annotations:
            annotations[BLOCK_PROPERTIES_KEY] = PersistentMapping({
                'view-name': u'block_view'
            })

        self.assertIn(u'My Text Block', get_block_html(block))



from collections import OrderedDict
from ftw.simplelayout import _
from ftw.simplelayout.browser.actions import DefaultActions
from ftw.simplelayout.contenttypes.contents.interfaces import IGalleryBlock
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.content import Container
from plone.directives import form
from zope import schema
from zope.i18n import translate
from zope.interface import alsoProvides
from zope.interface import implements
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


sort_index_vocabulary = SimpleVocabulary([
    SimpleTerm(
        value='sortable_title',
        title=_(u'label_title', default=u'Title')
    ),
    SimpleTerm(
        value='modified',
        title=_(u'label_modified', default=u'Modification date')
    ),
    SimpleTerm(
        value='id',
        title=_(u'label_id', default=u'ID')
    ),
    SimpleTerm(
        value='getObjPositionInParent',
        title=_(u'label_position_in_folder', default=u'Position in Folder')
    )
])

sort_order_vocabulary = SimpleVocabulary([
    SimpleTerm(value='ascending',
               title=_(u'label_ascending', default=u'Ascending')),
    SimpleTerm(value='descending',
               title=_(u'label_descending', default=u'Descending'))
])


class IGalleryBlockSchema(form.Schema):
    """GalleryBlock for simplelayout
    """

    title = schema.TextLine(
        title=_(u'label_title', default=u'Title'),
        required=False)

    show_title = schema.Bool(
        title=_(u'label_show_title', default=u'Show title'),
        default=True,
        required=False)

    sort_on = schema.Choice(
        title=_(u'label_sort_on', default=u'Sort by'),
        required=True,
        default="sortable_title",
        source=sort_index_vocabulary)

    sort_order = schema.Choice(
        title=_(u'label_sort_order', default=u'Sort order'),
        required=True,
        default="ascending",
        vocabulary=sort_order_vocabulary)


alsoProvides(IGalleryBlockSchema, IFormFieldProvider)


class GalleryBlock(Container):
    implements(IGalleryBlock)


class GalleryBlockActions(DefaultActions):

    def specific_actions(self):
        return OrderedDict([
            ('folderContents', {
                'class': 'sl-icon-folder-contents redirect',
                'title': translate(
                    _(u'label_folder_contents_images',
                      default=u'Go to folder contents for managing images'),
                    context=self.request),
                'href': '/folder_contents'
            }),
        ])



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.configuration import block_uids_in_page
from ftw.simplelayout.configuration import block_uids_missing_in_config
from ftw.simplelayout.configuration import columns_in_config
from ftw.simplelayout.configuration import flattened_block_uids
from ftw.simplelayout.configuration import synchronize_page_config_with_blocks
from ftw.simplelayout.interfaces import IBlockConfiguration
from ftw.simplelayout.interfaces import IPageConfiguration
from ftw.simplelayout.interfaces import ISimplelayoutContainerConfig
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING
from ftw.simplelayout.testing import SimplelayoutTestCase
from ftw.simplelayout.tests.sample_types import ISampleSimplelayoutContainer
from ftw.testing import staticuid
from zExceptions import Unauthorized
from zope.component import getGlobalSiteManager
from zope.interface import Interface
from zope.interface.verify import verifyObject


class TestPageConfiguration(SimplelayoutTestCase):
    layer = FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING

    def setUp(self):
        super(TestPageConfiguration, self).setUp()
        self.setup_sample_ftis(self.layer['portal'])

    def test_implements_interface(self):
        config = IPageConfiguration(create(Builder('sample container')))
        verifyObject(IPageConfiguration, config)

    def test_setting_and_loading_config(self):
        config = IPageConfiguration(create(Builder('sample container')))
        config.store(
            {'default': [{'cols': [{'blocks': [{'uid': 'foo'}]}]}]}
        )
        self.assertEquals(
            {'default': [{'cols': [{'blocks': [{'uid': 'foo'}]}]}]},
            config.load())

        config.store(
            {'default': [{'cols': [{'blocks': [{'uid': 'bar'}]}]}]}
        )
        self.assertEquals(
            {'default': [{'cols': [{'blocks': [{'uid': 'bar'}]}]}]},
            config.load())

    def test_config_is_recursive_persistent(self):
        config = IPageConfiguration(create(Builder('sample container')))
        config.store(
            {'default': [{'cols': [{'blocks': [{'uid': 'foo'}]}]}]}
        )
        self.assert_recursive_persistence(config.load())

    def test_default_config_is_recursive_persistent(self):
        config = IPageConfiguration(create(Builder('sample container')))
        self.assert_recursive_persistence(config.load())

    def test_restore_slot_default_when_removing_last_layout(self):
        """If the last layout was removed from a state, the slot default
        should be restored.
        """

        class ConfigAdapter(object):
            def __init__(self, context, request):
                pass

            def __call__(self, settings):
                pass

            def default_page_layout(self):
                return {'default': [{'cols': [{'blocks': [{'uid': 'foo'}]}]}]}

        adapter_registration_kwargs = {
            'factory': ConfigAdapter,
            'required': (ISampleSimplelayoutContainer, Interface),
            'provided': ISimplelayoutContainerConfig}

        container = create(Builder('sample container'))

        gsm = getGlobalSiteManager()
        gsm.registerAdapter(**adapter_registration_kwargs)
        try:
            config = IPageConfiguration(container)
            config.store(config.load())
            self.assertEquals(
                {'default': [{'cols': [{'blocks': [{'uid': 'foo'}]}]}]},
                config.load())

            config.store({'default': [{'cols': [{'blocks': []}]}],
                          'portlet': [{'cols': [{'blocks': [{'uid': 'bar'}]}]}]})
            self.assertEquals(
                {'default': [{'cols': [{'blocks': []}]}],
                 'portlet': [{'cols': [{'blocks': [{'uid': 'bar'}]}]}]},
                config.load())

            config.store({'default': []})
            self.assertEquals(
                {'default': [{'cols': [{'blocks': [{'uid': 'foo'}]}]}],
                 'portlet': [{'cols': [{'blocks': [{'uid': 'bar'}]}]}]},
                config.load())

        finally:
            gsm.unregisterAdapter(**adapter_registration_kwargs)

    def test_loaded_config_mutations_are_not_stored(self):
        config = IPageConfiguration(create(Builder('sample container')))
        config.store(
            {'default': [{'cols': [{'blocks': [{'uid': 'foo'}]}]}]}
        )

        config.load()['default'][0]['cols'][0]['blocks'].append(
            {'uid': 'bar'})

        self.assertEquals(
            {'default': [{'cols': [{'blocks': [{'uid': 'foo'}]}]}]},
            config.load())

    def test_unauthorized_when_not_allowed_to_change_layouts(self):
        page = create(Builder('sample container'))
        config = IPageConfiguration(page)
        config.store(
            {'default': [{'cols': [{'blocks': [{'uid': 'foo'},
                                               {'uid': 'bar'}]}]},
                         {'cols': [{'blocks': []},
                                   {'blocks': []}]}]}
        )

        # The user should not change layouts
        page.manage_permission('ftw.simplelayout: Change Layouts',
                               roles=[],
                               acquire=0)

        # When he changes layouts, Unauthorized is raised
        with self.assertRaises(Unauthorized):
            config.store(
                {'default': [{'cols': [{'blocks': [{'uid': 'foo'},
                                                   {'uid': 'bar'}]}]}]}
            )

        # But he is allowed to move blocks
        config.store(
            {'default': [{'cols': [{'blocks': [{'uid': 'foo'}]}]},
                         {'cols': [{'blocks': [{'uid': 'bar'}]},
                                   {'blocks': []}]}]}
        )


class TestPageConfigFunctions(SimplelayoutTestCase):
    layer = FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING

    def setUp(self):
        super(TestPageConfigFunctions, self).setUp()
        self.setup_sample_ftis(self.layer['portal'])

    def test_column_blocks_in_config(self):
        config = {'default': [{'cols': [{'blocks': [{'uid': 'foo'}]}]},
                              {'cols': [{'blocks': [{'uid': 'bar'}]},
                                        {'blocks': []}]}],
                  'sidebar': [{'cols': [{'blocks': [{'uid': 'baz'},
                                                    {'uid': 'foobar'}]}]}]}

        self.assertEquals(
            [{'blocks': [{'uid': 'foo'}]},
             {'blocks': [{'uid': 'bar'}]},
             {'blocks': []},
             {'blocks': [{'uid': 'baz'}, {'uid': 'foobar'}]}],
            columns_in_config(config))

    def test_flattened_block_uids(self):
        config = {'default': [{'cols': [{'blocks': [{'uid': 'foo'}]}]},
                              {'cols': [{'blocks': [{'uid': 'bar'}]},
                                        {'blocks': []}]}],
                  'sidebar': [{'cols': [{'blocks': [{'uid': 'baz'},
                                                    {'uid': 'foobar'}]}]}]}

        self.assertEquals(['foo', 'bar', 'baz', 'foobar'],
                          flattened_block_uids(config))

    @staticuid('staticuid')
    def test_block_uids_in_page(self):
        page = create(Builder('sample container'))
        create(Builder('sample block').within(page))
        create(Builder('sample block').within(page))

        self.assertEquals(
            ['staticuid00000000000000000000002',
             'staticuid00000000000000000000003'],
            block_uids_in_page(page))

    @staticuid('staticuid')
    def test_block_uids_missing_in_config(self):
        page = create(Builder('sample container'))
        create(Builder('sample block').within(page))
        self.assertEquals(
            ['staticuid00000000000000000000002'],
            block_uids_missing_in_config(page))

        IPageConfiguration(page).store(
            {'default': [{'cols': [{'blocks': [
                {'uid': 'staticuid00000000000000000000002'}]}]}]})

        create(Builder('sample block').within(page))
        create(Builder('sample block').within(page))

        self.assertEquals(
            ['staticuid00000000000000000000003',
             'staticuid00000000000000000000004'],
            block_uids_missing_in_config(page))

    @staticuid('staticuid')
    def test_synchronize_page_config_with_blocks(self):
        page = create(Builder('sample container'))

        create(Builder('sample block').within(page))
        IPageConfiguration(page).store(
            {'default': [{'cols': [{'blocks': [
                {'uid': 'staticuid00000000000000000000002'},
                {'uid': 'staticuid00000000000000000000004'}]}]}]})

        create(Builder('sample block').within(page))

        result = synchronize_page_config_with_blocks(page)
        self.assertEquals(
            {'added': ['staticuid00000000000000000000003'],
             'removed': ['staticuid00000000000000000000004']},
            result)

        self.assertEquals(
            {'default': [{'cols': [{'blocks': [
                {'uid': 'staticuid00000000000000000000002'},
                {'uid': 'staticuid00000000000000000000003'}]}]}]},
            IPageConfiguration(page).load())

    @staticuid('staticuid')
    def test_synchronize_page_config_with_blocks_on_empty_page(self):
        page = create(Builder('sample container'))
        create(Builder('sample block').within(page))
        result = synchronize_page_config_with_blocks(page)
        self.assertEquals(
            {'added': ['staticuid00000000000000000000002'],
             'removed': []},
            result)

        self.assertEquals(
            {'default': [{'cols': [{'blocks': [
                {'uid': 'staticuid00000000000000000000002'}]}]}]},
            IPageConfiguration(page).load())

    def test_store_partial_updates(self):
        page = create(Builder('sample container'))
        config = IPageConfiguration(page)

        state = {'default': [{'cols': [{'blocks': [{'uid': 'foo'}]}]},
                             {'cols': [{'blocks': [{'uid': 'bar'}]},
                                       {'blocks': []}]}],
                 'sidebar': [{'cols': [{'blocks': [{'uid': 'baz'},
                                                   {'uid': 'foobar'}]}]}]}
        config.store(state)

        partial_state = {'sidebar': [{'cols': [
            {'blocks': [{'uid': 'baz'}]}]}]}
        config.store(partial_state)

        # Remove block "foobar" from "sidebar" slot
        new_state = {'default': [{'cols': [{'blocks': [{'uid': 'foo'}]}]},
                                 {'cols': [{'blocks': [{'uid': 'bar'}]},
                                           {'blocks': []}]}],
                     'sidebar': [{'cols': [{'blocks': [{'uid': 'baz'}]}]}]}

        self.assertEquals(new_state, config.load())


class TestBlockConfiguration(SimplelayoutTestCase):
    layer = FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING

    def setUp(self):
        super(TestBlockConfiguration, self).setUp()
        self.setup_sample_ftis(self.layer['portal'])

    def test_implements_interface(self):
        config = IBlockConfiguration(create(Builder('sample block')))
        verifyObject(IBlockConfiguration, config)

    def test_config_is_recursive_persistent(self):
        config = IBlockConfiguration(create(Builder('sample block')))
        config.store({'scale': 'mini'})
        self.assert_recursive_persistence(config.load())



from ftw.builder import builder_registry
from ftw.builder import create
from ftw.builder.dexterity import DexterityBuilder
from ftw.simplelayout.configuration import synchronize_page_config_with_blocks
from operator import methodcaller
from path import Path
from plone.namedfile.file import NamedBlobImage


class ContenPageBuilder(DexterityBuilder):
    portal_type = 'ftw.simplelayout.ContentPage'

    def __init__(self, session):
        super(ContenPageBuilder, self).__init__(session)
        self.block_builders = []

    def with_blocks(self, *block_builders):
        self.block_builders.extend(block_builders)
        return self

    def after_create(self, obj):
        if self.block_builders:
            map(create, map(methodcaller('within', obj), self.block_builders))
            synchronize_page_config_with_blocks(obj)
        return super(ContenPageBuilder, self).after_create(obj)

builder_registry.register('sl content page', ContenPageBuilder)


class TextBlockBuilder(DexterityBuilder):
    portal_type = 'ftw.simplelayout.TextBlock'

    def with_dummy_image(self):
        image = Path(__file__).joinpath('..', 'assets', 'fullhd.jpg').abspath()
        self.arguments['image'] = NamedBlobImage(data=image.bytes(),
                                                 filename=u'test.gif')
        return self

    def with_cropped_image(self):
        image = Path(__file__).joinpath('..', 'assets', 'cropped.jpg').abspath()
        self.arguments['cropped_image'] = NamedBlobImage(data=image.bytes(),
                                                         filename=u'cropped.gif')
        return self

builder_registry.register('sl textblock', TextBlockBuilder)


class ListingBlockBuilder(DexterityBuilder):
    portal_type = 'ftw.simplelayout.FileListingBlock'

    def __init__(self, session):
        super(ListingBlockBuilder, self).__init__(session)
        self.children_builders = []

    def with_files(self, *file_builders):
        self.children_builders.extend(file_builders)
        return self

    def after_create(self, obj):
        map(create, map(methodcaller('within', obj), self.children_builders))
        return super(ListingBlockBuilder, self).after_create(obj)

builder_registry.register('sl listingblock', ListingBlockBuilder)


class MapBlockBuilder(DexterityBuilder):
    portal_type = 'ftw.simplelayout.MapBlock'

builder_registry.register('sl mapblock', MapBlockBuilder)


class VideoBlockBuilder(DexterityBuilder):
    portal_type = 'ftw.simplelayout.VideoBlock'

builder_registry.register('sl videoblock', VideoBlockBuilder)


class GalleryBlockBuilder(DexterityBuilder):
    portal_type = 'ftw.simplelayout.GalleryBlock'

builder_registry.register('sl galleryblock', GalleryBlockBuilder)


class AliasBlockBuilder(DexterityBuilder):
    portal_type = 'ftw.simplelayout.AliasBlock'

builder_registry.register('sl aliasblock', AliasBlockBuilder)



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.configuration import synchronize_page_config_with_blocks
from ftw.simplelayout.interfaces import IPageConfiguration
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_CONTENT_TESTING
from ftw.simplelayout.testing import SimplelayoutTestCase
from ftw.testbrowser import Browser
from plone.uuid.interfaces import IUUID
import transaction


class TestCopySimplelayoutPage(SimplelayoutTestCase):

    layer = FTW_SIMPLELAYOUT_CONTENT_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.setup_sample_ftis(self.portal)
        self.setup_block_views()

        self.page = create(Builder('sl content page'))
        self.block_left = create(Builder('sl textblock').within(self.page))
        self.block_right = create(Builder('sl textblock').within(self.page))
        self.block_below = create(Builder('sl textblock').within(self.page))

        self.page_state = {
            "default": [
                {"cols": [
                    {"blocks": [
                        {"uid": IUUID(self.block_left)}
                    ]},
                    {"blocks": [
                        {"uid": IUUID(self.block_right)}
                    ]},
                ]},
                {"cols": [
                    {"blocks": [
                        {"uid": IUUID(self.block_below)
                         }
                    ]}
                ]}
            ]
        }

        IPageConfiguration(self.page).store(self.page_state)
        transaction.commit()

    def assertPageLayout(self, page):
        browser = Browser()
        with browser(self.layer['app']):
            browser.login().visit(page)

            self.assertEquals(
                1,
                len(browser.css('.sl-layout:first-child .sl-column:first-child > .sl-block')),
                'Expect one block in first column of the first layout.')

            self.assertEquals(
                1,
                len(browser.css('.sl-layout:first-child .sl-column:nth-child(2) > .sl-block')),
                'Expect one block in second column of the first layout.')

            self.assertEquals(
                1,
                len(browser.css('.sl-layout:nth-child(2) .sl-block')),
                'Expect one block in second second layout.')

    def test_uids_in_block_state_are_updated_after_copy(self):
        copy = self.portal.manage_copyObjects([self.page.id])
        self.portal.manage_pasteObjects(copy)
        transaction.commit()
        self.assertPageLayout(self.page)

        newpage = self.portal.get('copy_of_' + self.page.id)
        self.assertPageLayout(newpage)

    def test_page_config_is_update_recursively(self):
        child_page = create(Builder('sl content page').within(self.page))
        create(Builder('sl textblock').within(child_page))
        create(Builder('sl textblock').within(child_page))
        synchronize_page_config_with_blocks(child_page)
        original_config = IPageConfiguration(child_page).load()

        clipboard = self.portal.manage_copyObjects(self.page.id)
        self.portal.manage_pasteObjects(clipboard)
        new_child = self.portal['copy_of_' + self.page.id][child_page.id]
        new_config = IPageConfiguration(new_child).load()

        self.assertNotEqual(original_config, new_config,
                            'The configs must be update recursively')



from Acquisition import aq_base
from Acquisition import aq_inner
from Acquisition import aq_parent
from BTrees.OOBTree import OOBTree
from contextlib import contextmanager
from copy import deepcopy
from DateTime import DateTime
from datetime import datetime
from ftw.simplelayout.configuration import columns_in_config
from ftw.simplelayout.interfaces import IBlockConfiguration
from ftw.simplelayout.interfaces import IPageConfiguration
from ftw.simplelayout.interfaces import ISimplelayoutBlock
from ftw.simplelayout.properties import BLOCK_PROPERTIES_KEY
from ftw.simplelayout.staging.interfaces import IBaseline
from ftw.simplelayout.staging.interfaces import IStaging
from ftw.simplelayout.staging.interfaces import IWorkingCopy
from ftw.simplelayout.utils import unrestricted_uuidToObject
from ftw.simplelayout.utils import IS_PLONE_5
from operator import methodcaller
from persistent.list import PersistentList
from persistent.mapping import PersistentMapping
from plone.app.textfield.interfaces import IRichText
from plone.app.textfield.value import RichTextValue
from plone.app.uuid.utils import uuidToObject
from plone.dexterity.interfaces import IDexterityContent
from plone.dexterity.interfaces import IDexterityFTI
from plone.dexterity.utils import getAdditionalSchemata
from plone.uuid.interfaces import IUUID
from Products.Archetypes.interfaces import IBaseContent
from zope.annotation.interfaces import IAnnotations
from zope.component import adapter
from zope.component import getGlobalSiteManager
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.interface import alsoProvides
from zope.interface import implementer
from zope.interface import Interface
from zope.interface import noLongerProvides
from zope.lifecycleevent.interfaces import IObjectCopiedEvent
from zope.schema import getFieldsInOrder
import pkg_resources

EVENT_SUPPORT = False
try:
    pkg_resources.get_distribution('plone.app.event')
except pkg_resources.DistributionNotFound:
    pass
else:
    if not IS_PLONE_5:
        EVENT_SUPPORT = True
        from plone.app.event.dx.behaviors import EventBasic


try:
    pkg_resources.get_distribution('ftw.trash')
except pkg_resources.DistributionNotFound:
    FTW_TRASH_SUPPORT = False
else:
    FTW_TRASH_SUPPORT = True
    from ftw.trash.interfaces import ITrashed


@implementer(IStaging)
@adapter(Interface)
def staging_lookup(context):
    """The staging lookup allows for an easy lookup of the adapter by
    simply using IStaging(context).
    We want the request to be adapted anyway, so that the adapter can be customized
    with a browser layer.
    """
    return getMultiAdapter((context, context.REQUEST), IStaging)


@implementer(IStaging)
@adapter(Interface, Interface)
class Staging(object):

    IGNORED_AT_FIELDS = (
        'id',
        'creation_date',
        'modification_date',
    )

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def is_baseline(self):
        """Returns whether the adapted object is a baseline.
        """
        return IBaseline.providedBy(self.context)

    def is_working_copy(self):
        """Returns whether the adapted object is a working copy.
        """
        return IWorkingCopy.providedBy(self.context)

    def get_baseline(self):
        """When the adapted object is a working copy, the baseline is returned,
        otherwise None.
        """
        if self.is_working_copy():
            return uuidToObject(self.context._baseline)

    def get_working_copies(self):
        """When the adapted object is a baseline, a list of working copies is returned.
        """
        if not self.is_baseline():
            return None

        return map(unrestricted_uuidToObject, self.context._working_copies)

    def create_working_copy(self, target_container):
        """Make a working copy of the adapted context into the given target container.
        """
        working_copy = self._create_clone(self.context, target_container)
        self._remove_auto_generated_children(self.context, working_copy)
        alsoProvides(self.context, IBaseline)
        alsoProvides(working_copy, IWorkingCopy)
        self._link(self.context, working_copy)
        uuid_map = self._map_uuids(self.context, working_copy)
        self._update_internal_links_recursively(working_copy, uuid_map)
        return working_copy

    def apply_working_copy(self):
        """Apply the changes of the working copy back to the baseline and remove
        the working copy.
        """
        if not self.is_working_copy():
            raise ValueError('Adapted context must be a working copy.')

        baseline = self.get_baseline()
        working_copy = self.context
        uuid_map = self._apply_children(working_copy, baseline,
                                        condition=self.is_child_integrated)
        self._update_internal_links_recursively(baseline, uuid_map,
                                                condition=self.is_child_integrated)

        self._update_simplelayout_page_state(working_copy, baseline, uuid_map)
        self._unlink_and_delete_working_copy(baseline, working_copy)

    def discard_working_copy(self):
        """When the adapted context is the working copy, this method discards the
        working copy by deleting it.
        """
        if not self.is_working_copy():
            raise ValueError('Adapted context must be a working copy.')

        baseline = self.get_baseline()
        working_copy = self.context
        self._unlink_and_delete_working_copy(baseline, working_copy)

    def is_child_integrated(self, obj):
        """Condition for deciding whether a child is "integrated" or not.
        By "integrated" it's meant that the child is considered part of the content
        and thus is copyied / applied along with the staged container.
        In simplelayout this applies to simplelayout blocks by defualt.
        """
        return ISimplelayoutBlock.providedBy(obj)

    def _create_clone(self, obj, target_container):
        """When cloning an object, we want to make sure that we do not clone
        child objects which we will not use later.
        This is important as we may make a working copy of a root node which
        contains very large structures.
        """
        source_container = aq_parent(aq_inner(obj))
        obj = aq_base(obj)
        with self._cleanup_filter_tree(obj), self._cleanup_filter_order(obj):
            clipboard = source_container.manage_copyObjects([obj.getId()])
            copy_info, = target_container.manage_pasteObjects(clipboard)
        working_copy = target_container.get(copy_info['new_id'])
        working_copy.creation_date = DateTime()
        working_copy.reindexObject(idxs=['created'])
        return working_copy

    def _remove_auto_generated_children(self, baseline, working_copy):
        """When creating a working copy, we do not want have additionaly auto generated
        children, which may be auto generated by subscribers.
        Reasons:
        - It will not work later in the staging process; it will cause an exception.
        - Each time we create a working copy we would end up with one child more.
        """
        baseline_children = set(baseline.objectIds())
        working_copy_children = set(working_copy.objectIds())
        children_to_delete = list(working_copy_children - baseline_children)

        if hasattr(working_copy, 'manage_immediatelyDeleteObjects'):
            # When ftw.trash is installed, immediately delete the working copy.
            working_copy.manage_immediatelyDeleteObjects(children_to_delete)
        else:
            working_copy.manage_delObjects(children_to_delete)

    @contextmanager
    def _cleanup_filter_tree(self, obj):
        """Filter the child objects of the ``obj`` so only children considered
        as "integrated" are beeing kept.
        The context manager restores on exit.
        """
        original = obj._tree
        obj._tree = OOBTree({key: value for (key, value) in obj._tree.items()
                             if self.is_child_integrated(value)})
        original_count = obj._count()
        obj._count.set(len(obj._tree))

        # Revert the tree patch within a IObjectCopiedEvent subscriber in order
        # to revert right after inserting the working copy object but before
        # other event subscribers are triggered.
        # One regular paste subscriber is from plone.app.linkitegrity, verifying
        # the relations. If we have relations with a subpage as target in our
        # pasted content, it will fail when the tree change is not reverted since
        # the subpage is then not reachable at all.
        def object_copied_subscriber(event):
            if event.original == obj:
                revert()

        handler_args = (object_copied_subscriber, (IObjectCopiedEvent,))

        def revert():
            if getattr(original, '_p_jar', object()) != getattr(obj, '_p_jar', object()):
                # The handler is registered for all thread; this is a call from another
                # thread, so ignore it.
                return

            if obj._tree == original:
                # revert() usually is called twice; only reset once.
                return
            getGlobalSiteManager().unregisterHandler(*handler_args)
            obj._tree = original
            obj._count.set(original_count)

        getGlobalSiteManager().registerHandler(*handler_args)
        try:
            yield
        finally:
            revert()

    @contextmanager
    def _cleanup_filter_order(self, obj):
        """Cleanup order according to already cleaned up object tree.
        The context manager restores on exit.
        """
        annotations = IAnnotations(obj)
        annkey = 'plone.folder.ordered.order'
        if annkey in annotations:
            original = annotations[annkey]
            annotations[annkey] = PersistentList(
                [key for key in original if key in obj._tree])
        try:
            yield
        finally:
            if annkey in annotations:
                annotations[annkey] = original

    def _map_uuids(self, baseline, working_copy):
        """Map uuids of baseline objects and working copy objects.
        The uuid of the baseline object is stored on the working copy object,
        recursively.
        The method returns a map from baseline objects to their working copy objects.
        """
        baseline_to_workingcopy = {}

        def handle(baseline_obj, working_copy_obj):
            baseline_to_workingcopy[IUUID(baseline_obj)] = IUUID(working_copy_obj)
            working_copy_obj._baseline_obj_uuid = IUUID(baseline_obj)
            for name in working_copy_obj.objectIds():
                handle(baseline_obj[name], working_copy_obj[name])

        handle(baseline, working_copy)
        return baseline_to_workingcopy

    def _link(self, baseline, working_copy):
        if not hasattr(baseline, '_working_copies'):
            baseline._working_copies = PersistentList()

        baseline._working_copies.append(IUUID(working_copy))
        working_copy._baseline = IUUID(baseline)

    def _unlink(self, baseline, working_copy):
        del working_copy._baseline
        baseline._working_copies.remove(IUUID(working_copy))

    def _unlink_and_delete_working_copy(self, baseline, working_copy):
        self._unlink(baseline, working_copy)
        noLongerProvides(baseline, IBaseline)
        noLongerProvides(working_copy, IWorkingCopy)

        parent = aq_parent(aq_inner(working_copy))
        if hasattr(parent, 'manage_immediatelyDeleteObjects'):
            # When ftw.trash is installed, immediately delete the working copy.
            parent.manage_immediatelyDeleteObjects([working_copy.getId()])
        else:
            parent.manage_delObjects([working_copy.getId()])

    def _apply_children(self, source, target, condition=None, uuid_map=None):
        uuid_map = uuid_map or {}
        uuid_map[IUUID(source)] = IUUID(target)
        target_children_map = {IUUID(obj): obj for obj in self._get_children(target, condition)}
        source_ids = source.objectIds()
        self._copy_field_values(source, target)
        self._purge_scales(target)
        self._update_simplelayout_block_state(source, target)

        for source_child in self._get_children(source, condition):
            target_uid = getattr(source_child, '_baseline_obj_uuid', None)
            if target_uid in target_children_map:
                target_child = target_children_map.pop(target_uid)
                self._apply_children(source_child, target_child, uuid_map=uuid_map)
            else:
                target_child = self._move_new_obj(source_child, target)
                uuid_map[IUUID(source_child)] = IUUID(target_child)

        target.moveObjectsToTop(source_ids)
        target.manage_delObjects(map(methodcaller('getId'), target_children_map.values()))
        return uuid_map

    def _update_simplelayout_page_state(self, working_copy, baseline, uuid_map):
        """Copy the simplelayout page state from the working copy to the baseline
        and change the working copy block UUIDs to the existing baseline block UUIDs
        unless new blocks are added.
        """
        config = IPageConfiguration(working_copy).load()
        for column in columns_in_config(config):
            for block in column['blocks']:
                if block['uid'] in uuid_map:
                    block['uid'] = uuid_map[block['uid']]

        IPageConfiguration(baseline).store(config)

    def _update_simplelayout_block_state(self, source, target):
        """Copy the simplelayout block state from the source block to the target block.
        """
        source_configuration = IBlockConfiguration(source, None)
        if source_configuration:
            config = deepcopy(source_configuration.load())
            IBlockConfiguration(target).store(config)

        source_ann = IAnnotations(source)
        target_ann = IAnnotations(target)
        if BLOCK_PROPERTIES_KEY in source_ann:
            target_ann[BLOCK_PROPERTIES_KEY] = PersistentMapping(
                source_ann[BLOCK_PROPERTIES_KEY])
        else:
            target_ann.pop(BLOCK_PROPERTIES_KEY, None)

    def _update_internal_links_recursively(self, obj, uuid_map, condition=None):
        if IDexterityContent.providedBy(obj):
            self._update_internal_links_DX(obj, uuid_map)
        for child in self._get_children(obj, condition):
            self._update_internal_links_recursively(child, uuid_map)

    def _update_internal_links_DX(self, obj, uuid_map):
        for name, field, schemata in self._iter_fields(obj.portal_type):
            if not IRichText.providedBy(field):
                continue

            storage = schemata(obj)
            ori_text = getattr(storage, name)
            if not ori_text:
                continue

            new_text = self._replace_uids_in_html(ori_text.raw, uuid_map)
            if new_text == ori_text.raw:
                continue

            setattr(storage, name, RichTextValue(new_text,
                                                 mimeType=ori_text._mimeType,
                                                 outputMimeType=ori_text._outputMimeType))

    def _replace_uids_in_html(self, html, uuid_map):
        for from_uuid, to_uuid in uuid_map.items():
            html = html.replace(from_uuid, to_uuid)
        return html

    def _copy_field_values(self, source, target):
        """Copy all fields values from "source" to "target".
        """
        if type(source) != type(target):
            raise ValueError('Objects have differing classes ({!r} != {!r})'.format(
                type(source), type(target)))

        if IDexterityContent.providedBy(source):
            return self._copy_dx_field_values(source, target)

        if IBaseContent.providedBy(source):
            return self._copy_at_field_values(source, target)

        raise ValueError('Unsupported object type {!r}, neither AT nor DX.'.format(
            source))

    def _copy_dx_field_values(self, source, target):
        for name, field, schemata in self._iter_fields(source.portal_type):
            if name == 'id':
                # never re-set the id.
                continue
            source_storage = schemata(source)
            target_storage = schemata(target)
            value = getattr(source_storage, name)

            if EVENT_SUPPORT and isinstance(target_storage, EventBasic) and \
               isinstance(value, datetime):
                value = getattr(source, name)
            elif isinstance(value, str):
                value = value.decode('utf-8')

            setattr(target_storage, field.getName(), value)

    def _copy_at_field_values(self, source, target):
        for source_field in source.Schema().values():
            if source_field.__name__ in self.IGNORED_AT_FIELDS:
                continue

            # Re-fetch the field from the target schema since source and
            # target schema may vary because of dynamice schema extenders.
            target_field = target.Schema().getField(source_field.__name__)
            value = source_field.getRaw(source)
            target_field.set(target, value)

    def _iter_fields(self, portal_type):
        for schemata in self._iter_schemata_for_protal_type(portal_type):
            for name, field in getFieldsInOrder(schemata):
                if not getattr(field, 'readonly', False):
                    yield (name, field, schemata)

    def _iter_schemata_for_protal_type(self, portal_type):
        fti = getUtility(IDexterityFTI, name=portal_type)
        yield fti.lookupSchema()
        for schema in getAdditionalSchemata(portal_type=portal_type):
            yield schema

    def _move_new_obj(self, obj, new_parent):
        clipboard = aq_parent(aq_inner(obj)).manage_cutObjects([obj.getId()])
        info = new_parent.manage_pasteObjects(clipboard)
        return new_parent.get(info[0]['new_id'])

    def _purge_scales(self, obj):
        """When copying value from one object to another, they may include images
        which have scales. In order to make sure that we do not use old caches,
        we are purging the complete scaling cache so that the scales will be
        regenerated when needed.
        """
        IAnnotations(obj).pop('plone.scale', None)

    def _get_children(self, folder, filter_condition=None):
        """Return the children of a container, supporting the trash and an arbitrary
        filter condition.
        """
        children = filter(filter_condition, folder.objectValues())

        if FTW_TRASH_SUPPORT:
            children = filter(lambda item: not ITrashed.providedBy(item), children)

        return children



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.browser.provider import SimplelayoutRenderer
from ftw.simplelayout.browser.simplelayout import SimplelayoutView
from ftw.simplelayout.interfaces import IBlockProperties
from ftw.simplelayout.interfaces import IPageConfiguration
from ftw.simplelayout.interfaces import ISimplelayoutContainerConfig
from ftw.simplelayout.interfaces import ISimplelayoutDefaultSettings
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING
from ftw.simplelayout.testing import SimplelayoutTestCase
from ftw.simplelayout.tests.sample_types import ISampleSimplelayoutContainer
from ftw.simplelayout.utils import IS_PLONE_5
from ftw.testbrowser import browsing
from plone.app.textfield.value import RichTextValue
from plone.registry.interfaces import IRegistry
from plone.uuid.interfaces import IUUID
from zExceptions import BadRequest
from zExceptions import Unauthorized
from zope.component import getGlobalSiteManager
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component import provideAdapter
from zope.interface import implements
from zope.interface import Interface
from zope.publisher.interfaces.browser import IBrowserView
import json
import transaction


class TestSimplelayoutView(SimplelayoutTestCase):

    layer = FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING

    def setUp(self):
        super(TestSimplelayoutView, self).setUp()
        self.setup_sample_ftis(self.layer['portal'])
        self.setup_block_views()
        self.container = create(Builder('sample container'))
        self.page_config = IPageConfiguration(self.container)
        self.url = self.container.absolute_url() + '/@@simplelayout-view'

        self.payload = {
            "default": [
                {
                    "cols": [
                        {
                            "blocks": [
                                {
                                    "uid": "c774b0ca2a5544bf9bb46d865b11bff9"
                                }
                            ]
                        }
                    ]
                },
                {
                    "cols": [
                        {
                            "blocks": [
                                {
                                    "uid": "413fb945952d4403a58ab1958c38f1d2"
                                }
                            ]
                        }
                    ]
                }
            ]
        }

    @browsing
    def test_render_blocks_not_in_page_configuration(self, browser):
        # Fallback for not saved blocks thru the simplelayout JS lib.
        create(Builder('sample block')
               .titled('TextBlock title')
               .within(self.container)
               .having(text=RichTextValue('The text'))
               )

        browser.login().visit(self.container, view='@@simplelayout-view')

        self.assertEqual(browser.url, self.url)
        self.assertEquals('OK',
                          browser.css('.sl-block').first.text)

    @browsing
    def test_invalid_simplelayout_save_state_request(self, browser):
        with browser.expect_http_error(reason='Bad Request'):
            browser.login().visit(self.container,
                                  view='sl-ajax-save-state-view',
                                  data={})

    @browsing
    def test_store_save_simplelayout_state_thru_view(self, browser):
        payload = {"data": json.dumps(self.payload)}

        if IS_PLONE_5:
            from plone.protect.authenticator import createToken
            payload["_authenticator"] = createToken()

        browser.login().visit(self.container,
                              view='sl-ajax-save-state-view',
                              data=payload)

        browser.visit(self.container)
        self.assertEquals(self.payload, self.page_config.load())

    @browsing
    def test_render_blocks_in_different_layouts(self, browser):
        block1 = create(Builder('sample block')
                        .titled('Block 1')
                        .within(self.container))
        block2 = create(Builder('sample block')
                        .titled('Block 1')
                        .within(self.container))

        self.payload['default'][0]['cols'][0][
            'blocks'][0]['uid'] = IUUID(block1)
        self.payload['default'][1]['cols'][0][
            'blocks'][0]['uid'] = IUUID(block2)
        self.page_config.store(self.payload)
        transaction.commit()

        browser.login().visit(self.container)
        self.assertEquals(2,
                          len(browser.css('.sl-layout')),
                          'Expect 2 layouts')

        self.assertEquals(2,
                          len(browser.css('.sl-column.sl-col-1')),
                          'Expect two, one column layouts')

    @browsing
    def test_render_blocks_in_different_columns(self, browser):
        block1 = create(Builder('sample block')
                        .titled('Block 1')
                        .within(self.container))
        block2 = create(Builder('sample block')
                        .titled('Block 1')
                        .within(self.container))

        self.payload['default'][0]['cols'][0][
            'blocks'][0]['uid'] = IUUID(block1)
        self.payload['default'][1]['cols'][0][
            'blocks'][0]['uid'] = IUUID(block2)

        # Move Block into layout 1, column 2
        data_colmn = self.payload['default'][1]['cols'][0]
        self.payload['default'].pop()

        self.payload['default'][0]['cols'].append(data_colmn)
        self.page_config.store(self.payload)
        transaction.commit()

        browser.login().visit(self.container)
        self.assertEquals(2,
                          len(browser.css('.sl-column.sl-col-2')),
                          'Expect 2 columns')

    @browsing
    def test_skips_unavailable_blocks(self, browser):
        # The block may no longer exist or may not be visible for
        # the current user.

        block1 = create(Builder('sample block')
                        .titled('Block 1')
                        .within(self.container))

        self.payload['default'][0]['cols'][0][
            'blocks'][0]['uid'] = IUUID(block1)
        self.payload['default'][1]['cols'][0][
            'blocks'][0]['uid'] = '123xNONxEXISTINGxUIDx123'
        self.page_config.store(self.payload)
        transaction.commit()

        browser.login().visit(self.container)
        self.assertEquals(
            ['http://nohost/plone/samplecontainer/block-1'],
            map(lambda node: node.attrib.get('data-url'),
                browser.css('.sl-block')))

    @browsing
    def test_empty_block_state_does_not_break_the_view(self, browser):
        self.payload['default'][0]['cols'][0]['blocks'].append({})
        self.page_config.store(self.payload)
        transaction.commit()

        browser.login().visit(self.container)
        self.assertTrue(
            browser.css('body.template-simplelayout-view'),
            'Expect to be on the simplelayout template, not the error page.')

    @browsing
    def test_simplelayout_default_config_from_control_panel(self, browser):
        browser.login().visit(self.container, view='@@simplelayout-view')

        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISimplelayoutDefaultSettings)
        settings.slconfig = u'{"layouts": [1, 2]}'
        transaction.commit()

        browser.login().visit(self.container, view='@@simplelayout-view')
        data_attr_value = json.loads(browser.css(
            '[data-sl-settings]').first.attrib['data-sl-settings'])

        self.assertEquals([1, 2],
                          data_attr_value['layouts'],
                          'Expect the layout setting in default config.')

    @browsing
    def test_simplelayout_config_updated_by_permissions(self, browser):

        browser.login().visit(self.container, view='@@simplelayout-view')
        data_attr_value = json.loads(browser.css(
            '[data-sl-settings]').first.attrib['data-sl-settings'])

        self.assertTrue(data_attr_value['canChangeLayout'],
                        'Should have the Change layouts permission.')

        self.container.manage_permission('ftw.simplelayout: Change Layouts',
                                         roles=[],
                                         acquire=0)
        transaction.commit()

        browser.visit(self.container, view='@@simplelayout-view')
        data_attr_value = json.loads(browser.css(
            '[data-sl-settings]').first.attrib['data-sl-settings'])

        self.assertFalse(data_attr_value['canChangeLayout'],
                         'Should NOT have the Change layouts permission.')

    @browsing
    def test_prevent_layout_changes_if_not_allowed(self, browser):

        self.container.manage_permission('ftw.simplelayout: Change Layouts',
                                         roles=[],
                                         acquire=0)
        transaction.commit()

        self.payload['default'].append({'cols': [{}]})
        payload = {"data": json.dumps(self.payload)}
        with browser.expect_unauthorized():
            browser.login().visit(self.container,
                                  view='sl-ajax-save-state-view',
                                  data=payload)

    @browsing
    def test_simplelayout_config_updated_by_adapter(self, browser):

        class ContainerConfigAdapter(object):
            implements(ISimplelayoutContainerConfig)

            def __init__(self, context, request):
                pass

            def __call__(self, settings):
                settings['layouts'] = [1]

            def default_page_layout(self):
                return None

        provideAdapter(ContainerConfigAdapter,
                       adapts=(ISampleSimplelayoutContainer, Interface))
        transaction.commit()

        try:
            browser.login().visit(self.container, view='@@simplelayout-view')
            data_attr_value = json.loads(browser.css(
                '[data-sl-settings]').first.attrib['data-sl-settings'])
            self.assertEquals([1], data_attr_value['layouts'])

        finally:
            # Unregister adapter - since the component registry is not isolatet
            # per test
            sm = getGlobalSiteManager()
            sm.unregisterAdapter(ContainerConfigAdapter,
                                 required=(
                                     ISampleSimplelayoutContainer, Interface),
                                 provided=ISimplelayoutContainerConfig)

    @browsing
    def test_simplelayout_default_page_layouts_by_adapter(self, browser):

        class ContainerConfigAdapter(object):
            implements(ISimplelayoutContainerConfig)

            def __init__(self, context, request):
                pass

            def __call__(self, settings):
                pass

            def default_page_layout(self):
                return {
                    "default": [
                        {"cols": [{"blocks": []},
                                  {"blocks": []}]}
                    ]
                }

        provideAdapter(ContainerConfigAdapter,
                       adapts=(ISampleSimplelayoutContainer, Interface))
        transaction.commit()

        try:
            browser.login().visit(self.container, view='@@simplelayout-view')
            # This should result in one layout with two columns
            self.assertEquals(1, len(browser.css('.sl-layout')))
            self.assertEquals(2, len(browser.css('.sl-column.sl-col-2')))

        finally:
            # Unregister adapter - since the component registry is not isolatet
            # per test
            sm = getGlobalSiteManager()
            sm.unregisterAdapter(ContainerConfigAdapter,
                                 required=(
                                     ISampleSimplelayoutContainer, Interface),
                                 provided=ISimplelayoutContainerConfig)

    @browsing
    def test_simplelayout_config_updated_view(self, browser):

        class CustomSimplelayoutView(SimplelayoutView):
            def update_simplelayout_settings(self, settings):
                settings['layouts'] = [1, 4]

        provideAdapter(CustomSimplelayoutView,
                       adapts=(Interface, Interface),
                       provides=IBrowserView,
                       name='customview')

        browser.login().visit(self.container, view='@@customview')
        data_attr_value = json.loads(browser.css(
            '[data-sl-settings]').first.attrib['data-sl-settings'])
        self.assertEquals([1, 4], data_attr_value['layouts'])

    @browsing
    def test_show_fallback_view_on_block_render_problems(self, browser):
        block = create(Builder('sample block')
                       .titled('TextBlock title')
                       .within(self.container))

        properties = getMultiAdapter((block, block.REQUEST),
                                     IBlockProperties)
        properties.set_view('block_view_broken')

        transaction.commit()

        browser.login().visit(self.container)
        self.assertEquals(
            'The block could not be rendered. Please check the log for '
            'details.',
            browser.css('.sl-block').first.text)

    @browsing
    def test_empty_sl_page_renders_at_least_one_layout(self, browser):
        browser.login().visit(self.container)

        # By default it's a one column layout.
        self.assertEquals(1,
                          len(browser.css('.sl-column.sl-col-1')),
                          'There should be at least a empty one column layout')

    @browsing
    def test_normalized_portal_type_as_css_klass_on_block(self, browser):
        create(Builder('sample block')
               .titled('TextBlock title')
               .within(self.container))
        browser.login().visit(self.container)
        self.assertEquals('sl-block sampleblock',
                          browser.css('.sl-block').first.attrib['class'],
                          'Expect "sample" as css klass on block structure.')

    @browsing
    def test_block_has_anchor(self, browser):
        create(Builder('sample block')
               .titled('Block 1')
               .within(self.container))

        browser.login().open(self.container)
        self.assertEqual(
            'block-1',
            browser.css('.sl-layout a').first.attrib['name']
        )

    @browsing
    def test_canEdit_is_false_if_border_disabled(self, browser):
        browser.login().visit(self.container, data={'disable_border': 1})
        data_attr_value = json.loads(browser.css(
            '[data-sl-settings]').first.attrib['data-sl-settings'])

        self.assertFalse(data_attr_value['canEdit'],
                         'Edit should be disabled if disable_border is there.')

    @browsing
    def test_sl_container_has_sl_edit_css_class(self, browser):
        browser.login().visit(self.container)
        self.assertTrue(len(browser.css('.sl-can-edit')),
                        'Expect the sl-can-edit class on sl-simplelayout.')

    @browsing
    def test_sl_container_no_sl_edi_css_class(self, browser):
        browser.login().visit(self.container, data={'disable_border': 1})
        self.assertFalse(len(browser.css('.sl-can-edit')),
                         'no sl-can-edit class expected.')

    @browsing
    def test_simplelayout_renderer_only_one_layout_of_storage(self, browser):
        storage = self.payload.copy()
        sl_renderer = SimplelayoutRenderer(self.container, storage, 'default')

        browser.parse(sl_renderer.render_layout(index=1))
        self.assertEquals(1, len(browser.css('.sl-layout')),
                          'Expect only one layout')

        browser.parse(sl_renderer.render_layout(index=0))
        self.assertEquals(1, len(browser.css('.sl-layout')),
                          'Expect only one layout')

        browser.parse(sl_renderer.render_layout())
        self.assertEquals(2, len(browser.css('.sl-layout')),
                          'Expect both layouts')

    @browsing
    def test_simplelayout_renderer_raises_ValueError(self, browser):
        storage = self.payload.copy()
        sl_renderer = SimplelayoutRenderer(self.container, storage, 'default')

        with self.assertRaises(ValueError):
            sl_renderer.render_layout(index=4)

    @browsing
    def test_simplelayout_renderer_can_omit_sl_layout_class(self, browser):
        storage = self.payload.copy()
        sl_renderer = SimplelayoutRenderer(self.container, storage, 'default')

        browser.open_html(sl_renderer.render_layout(index=0, is_update=True))
        self.assertEquals(0, len(browser.css('.sl-layout')),
                          'Expect no sl-layout wrapper')



from ftw.simplelayout import _
from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider
from plone.namedfile.field import NamedBlobImage
from plone.supermodel import model
from zope import schema
from zope.interface import provider


@provider(IFormFieldProvider)
class ICroppedImageInOverlay(model.Schema):
    model.fieldset('image',
                   fields=['use_cropped_image_for_overlay'])

    use_cropped_image_for_overlay = schema.Bool(
        title=_(u'use_cropped_image_for_overlay',
                default=u'Use cropped image for overlay'),
        description=_(u'use_cropped_image_for_overlay_desc',
                      default=u'If you crop an image, the original image will be still available. '
                              u'Per default, the original image will be used for the overlay. '
                              u'If you want to show the cropped image instead, mark this option.'),
        required=False,
        default=True,
        missing_value=True)


@provider(IFormFieldProvider)
class IImageCropping(model.Schema):

    # This field should be hidden. Unfortunately, that's not possible due to a
    # weird bug in the file-selector-widget.
    # If this field is hidden and contains an image, it will raise a validation
    # error on form-save. If we skip the validation of this field, the image will
    # be broken after saving. So, we just hide this field through css.
    #
    # WARNING: This case is not testet. I can't reproduce this issue in the test.
    #
    # form.mode(cropped_config='hidden')
    cropped_image = NamedBlobImage(
        title=_(u'label_cropped_image', default=u'Cropped Image'),
        required=False)

    form.mode(cropped_config='hidden')
    cropped_config = schema.TextLine(
        title=_(u'label_cropped_config', default=u'The final cropped configuration'),
        required=False)



from collections import OrderedDict
from collective import dexteritytextindexer
from ftw.simplelayout import _
from ftw.simplelayout.browser.actions import DefaultActions
from ftw.simplelayout.contenttypes.contents.interfaces import ITextBlock
from ftw.simplelayout.images.limits.validators import ImageLimitValidator
from ftw.simplelayout.interfaces import IBlockConfiguration
from ftw.simplelayout.interfaces import IBlockModifier
from plone.app.textfield import RichText
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.content import Item
from plone.directives import form
from plone.namedfile.field import NamedBlobImage
from z3c.form import validator
from zope import schema
from zope.i18n import translate
from zope.interface import alsoProvides
from zope.interface import implements


class ITextBlockSchema(form.Schema):

    """TextBlock for simplelayout
    """

    form.fieldset('image',
                  label=_(u'Image'),
                  fields=['image', 'image_alt_text', 'image_caption',
                          'open_image_in_overlay']
                  )

    title = schema.TextLine(
        title=_(u'label_title', default=u'Title'),
        required=False)

    show_title = schema.Bool(
        title=_(u'label_show_title', default=u'Show title'),
        default=True,
        required=False)

    dexteritytextindexer.searchable('text')
    form.primary('text')
    text = RichText(
        title=_(u'label_text', default=u'Text'),
        required=False,
        allowed_mime_types=('text/html',))

    image = NamedBlobImage(
        title=_(u'label_image', default=u'Image'),
        required=False)

    image_alt_text = schema.TextLine(
        title=_(u'label_image_alt_text', default=u'Image alternative text'),
        required=False,
        description=_(u'description_image_alt_text',
                      default=u'Enter an alternative text for the image'))

    image_caption = schema.TextLine(
        title=_(u'label_image_caption', default=u'Image caption'),
        required=False)

    open_image_in_overlay = schema.Bool(
        title=_(u'label_open_image_in_overlay',
                default=u'Open image in overlay'
                u' (only if there is no teaser url)'),
        default=False,
        required=False,
        description=_(u'description_image_clickable',
                      default=u'Opens image in an overlay'))


alsoProvides(ITextBlockSchema, IFormFieldProvider)


class TextBlockImageLimitValidator(ImageLimitValidator):

    identifier = 'ftw.simplelayout.TextBlock'


validator.WidgetValidatorDiscriminators(
    TextBlockImageLimitValidator,
    field=ITextBlockSchema['image']
)


class TextBlock(Item):
    implements(ITextBlock)

    @property
    def additional_css_classes(self):
        if self.title and not (self.image or self.text):
            return ['titleOnly']
        else:
            return []


class TextBlockModifier(object):

    implements(IBlockModifier)

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def modify(self, data):
        image_scale = data.get('scale', None)
        image_float = data.get('imagefloat', None)
        conf = IBlockConfiguration(self.context)
        blockconf = conf.load()

        if image_scale:
            blockconf['scale'] = image_scale
            blockconf['imagefloat'] = image_float
            conf.store(blockconf)  # necessary?
        return


class TextBlockActions(DefaultActions):

    def specific_actions(self):
        return OrderedDict([
            ('imageLeft', {
                'class': 'sl-icon-image-left block-server-action',
                'title': translate(
                    _(u'label_float_image_left', default=u'Float image left'),
                    context=self.request),
                'href': './sl-ajax-reload-block-view',
                'data-scale': 'sl_textblock_small',
                'data-imagefloat': 'left'
            }),
            ('imageLeftLarge', {
                'class': 'sl-icon-image-left-large block-server-action',
                'title': translate(
                    _(u'label_float_large_image_left',
                      default=u'Float large image left'),
                    context=self.request),
                'href': './sl-ajax-reload-block-view',
                'data-scale': 'sl_textblock_middle',
                'data-imagefloat': 'left'
            }),
            ('image', {
                'class': 'sl-icon-image block-server-action',
                'title': translate(
                    _(u'label_image_without_floating',
                      default=u'Image without floating'),
                    context=self.request),
                'href': './sl-ajax-reload-block-view',
                'data-scale': 'sl_textblock_large',
                'data-imagefloat': 'no-float'
            }),
            ('imageRightLarge', {
                'class': 'sl-icon-image-right-large block-server-action',
                'title': translate(
                    _(u'label_float_large_image_right',
                      default=u'Float large image right'),
                    context=self.request),
                'href': './sl-ajax-reload-block-view',
                'data-scale': 'sl_textblock_middle',
                'data-imagefloat': 'right'
            }),
            ('imageRight', {
                'class': 'sl-icon-image-right block-server-action',
                'title': translate(
                    _(u'label_float_image_right',
                      default=u'Float image right'),
                    context=self.request),
                'href': './sl-ajax-reload-block-view',
                'data-scale': 'sl_textblock_small',
                'data-imagefloat': 'right'
            }),
            ('cropping', {
                'class': 'sl-icon-crop crop-image',
                'title': translate(
                    _(u'label_crop_image', default='Crop image'),
                    context=self.request),
                'href': './sl-ajax-crop-image'
            }),
        ])



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_CONTENT_TESTING
from ftw.simplelayout.utils import IS_PLONE_5
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import factoriesmenu
from ftw.testbrowser.pages import plone
from unittest import skipIf
from unittest import TestCase


class TestSampleTypes(TestCase):

    layer = FTW_SIMPLELAYOUT_CONTENT_TESTING

    def setUp(self):
        super(TestSampleTypes, self).setUp()

        self.page = create(Builder('sl content page').titled(u'A page'))

    @browsing
    def test_add_page(self, browser):
        # Page
        browser.login().visit()
        factoriesmenu.add('ContentPage')
        browser.fill({'Title': u'A test page'})
        browser.find_button_by_label('Save').click()

        self.assertEquals('A test page', plone.first_heading())

    @browsing
    def test_add_textblock_with_title(self, browser):
        browser.login().visit(self.page)
        factoriesmenu.add('TextBlock')
        browser.fill({'Title': u'A test textblock',
                      'Text': u'Some text',
                      'Show title': True})
        browser.find_button_by_label('Save').click()

        self.assertIn('Some text',
                      browser.css('body').first.text)
        self.assertEquals('A test textblock',
                          browser.css('h2').first.text)

    @browsing
    def test_add_textblock_without_title(self, browser):
        browser.login().visit(self.page)
        factoriesmenu.add('TextBlock')
        browser.fill({'Title': u'A test textblock',
                      'Text': u'Some text',
                      'Show title': False})
        browser.find_button_by_label('Save').click()

        self.assertIn('Some text',
                      browser.css('body').first.text)
        self.assertEquals(0,
                          len(browser.css('h2')),
                          'Expect no textblock title')

    @browsing
    def test_nested_contentpages(self, browser):
        nested = create(Builder('sl content page')
                        .titled(u'Nested')
                        .within(self.page))

        browser.login().visit(self.page)

        self.assertFalse(browser.css('.sl-block'),
                         'Expect no block, also the contentpage should not be '
                         'visible as block')

        browser.visit(nested)
        self.assertEquals('Nested', plone.first_heading())

    @browsing
    def test_add_file_to_listingblock(self, browser):
        listingblock = create(Builder('sl listingblock')
                              .titled('ListingBlock')
                              .within(self.page))

        browser.login().visit(listingblock, view='folder_contents')
        factoriesmenu.add('File')
        browser.fill(
            {'File': ('Some Data', 'file.txt', 'text/plain')})
        browser.find_button_by_label('Save').click()

        self.assertEquals(
            '{0}/file.txt/view'.format(listingblock.absolute_url()),
            browser.url)


    @skipIf(IS_PLONE_5, 'No mapblock on plone 5.1')
    @browsing
    def test_adding_mapblock(self, browser):
        browser.login().visit(self.page)
        factoriesmenu.add('MapBlock')
        browser.fill({'form.widgets.IBlockCoordinates.coordinates': 'POINT(7.444608499999999 46.9479222)'})
        browser.find_button_by_label('Save').click()
        self.assertEquals(1, len(browser.css('.ftw-simplelayout-mapblock')))

    @skipIf(IS_PLONE_5, 'No mapblock on plone 5.1')
    @browsing
    def test_mapblock_id_based_on_block_id(self, browser):
        mapblock = create(Builder('sl mapblock').within(self.page))
        browser.login().visit(self.page)
        self.assertTrue(browser.css('#geo-{0}'.format(mapblock.id)),
                        'Did not found any mablock')



from ftw.simplelayout import _
from ftw.simplelayout.images.interfaces import IImageLimits
from ftw.simplelayout.images.interfaces import IImageLimitValidatorMessages
from Products.Five.browser import BrowserView


class LimitIndicator(BrowserView):

    limit_type = None

    def __init__(self, context, request):
        super(LimitIndicator, self).__init__(context, request)
        self.image_limits = IImageLimits(self.context)
        self.limit_validator_messages = IImageLimitValidatorMessages(self.context)

    def __call__(self):
        if not self.image_limits.validate('hard'):
            self.limit_type = 'hard'
        elif not self.image_limits.validate('soft'):
            self.limit_type = 'soft'

        return super(LimitIndicator, self).__call__()

    def title(self):
        if self.limit_type == 'hard':
            return _('imageresolution_too_small_label',
                     default="imageresolution is too small")
        elif self.limit_type == 'soft':
            return _('low_quality_image_label', default="low image quality")

    def description(self):
        return self.limit_validator_messages.limit_not_satisfied_message(
            self.limit_type)

    def show_indicator(self):
        return self.limit_type is not None

    def css_klass(self):
        classes = ['limitIndicator']

        if self.limit_type is not None:
            classes.append('{}LimitIndicator'.format(self.limit_type))

        return ' '.join(classes)



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.interfaces import IPageConfiguration
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING
from ftw.simplelayout.testing import SimplelayoutTestCase
from ftw.testbrowser import browsing
from zExceptions import BadRequest
import json


class TestLayoutReloadView(SimplelayoutTestCase):

    layer = FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.setup_sample_ftis(self.portal)
        self.setup_block_views()
        self.content = create(Builder('sample container'))

    def test_layout_reload_view_needs_slot_name_and_layoutindex(self):
        with self.assertRaises(BadRequest):
            self.content.restrictedTraverse('@@sl-ajax-reload-layout-view')()

        payload = json.dumps({'name': 'default', 'layoutindex': '0'})
        self.portal.REQUEST.set('data', payload)
        response = json.loads(self.content.restrictedTraverse(
            '@@sl-ajax-reload-layout-view')())

        self.assertIn('content', response)

    @browsing
    def test_layout_reload_view_renders_layout_without_wrapper(self, browser):
        payload = json.dumps({'name': 'default', 'layoutindex': '0'})
        self.portal.REQUEST.set('data', payload)
        response = json.loads(self.content.restrictedTraverse(
            '@@sl-ajax-reload-layout-view')())

        browser.open_html(response['content'])

        self.assertFalse(browser.css('.sl-layout'),
                         'Expect no sl-layout wrapper')

    @browsing
    def test_column_has_empty_config_by_default(self, browser):
        payload = json.dumps({'name': 'default', 'layoutindex': '0'})
        self.portal.REQUEST.set('data', payload)
        response = json.loads(self.content.restrictedTraverse(
            '@@sl-ajax-reload-layout-view')())

        browser.open_html(response['content'])

        self.assertEquals(
            '{}', browser.css('.sl-layout-content').first.attrib['data-config']
        )

    @browsing
    def test_render_store_layout_configuration_in_state(self, browser):
        config = IPageConfiguration(self.content)
        state = {
            "default": [
                {"cols": [{"blocks": []}]},
                {"cols": [{"blocks": []}]}
            ]
        }
        config.store(state)

        configdata = {'somekey': 'somevalue'}
        payload = json.dumps({'name': 'default',
                              'layoutindex': '1',
                              'config': configdata})
        self.portal.REQUEST.set('data', payload)
        response = json.loads(self.content.restrictedTraverse(
            '@@sl-ajax-reload-layout-view')())

        browser.open_html(response['content'])

        self.assertDictEqual(
            configdata,
            json.loads(
                browser.css('.sl-layout-content').first.attrib['data-config']))
        self.assertTrue(browser.css('.sl-layout-content.somevalue'),
                        'Expect one item')



from ftw.simplelayout.utils import IS_PLONE_5
from plone.registry.interfaces import IRegistry
from zope.component import getUtility


def profile_uninstalled(site):
    clean_plone5_registry(site)


def clean_plone5_registry(site):
    registry = getUtility(IRegistry)

    allowed_sizes = registry['plone.allowed_sizes']
    # allowed_sizes.remove(u'colorbox 2000:2000')
    allowed_sizes.remove(u'simplelayout_galleryblock 480:480')
    allowed_sizes.remove(u'sl_galleryblock_4k 3840:2160')
    allowed_sizes.remove(u'sl_textblock_small 480:480')
    allowed_sizes.remove(u'sl_textblock_middle 800:800')
    allowed_sizes.remove(u'sl_textblock_large 1280:1280')
    registry['plone.allowed_sizes'] = allowed_sizes

    types_not_searched = list(registry['plone.types_not_searched'])
    types_not_searched.remove('ftw.simplelayout.TextBlock')
    types_not_searched.remove('ftw.simplelayout.FileListingBlock')
    types_not_searched.remove('ftw.simplelayout.VideoBlock')
    types_not_searched.remove('ftw.simplelayout.GalleryBlock')
    registry['plone.types_not_searched'] = tuple(types_not_searched)

    default_page_types = registry['plone.default_page_types']
    default_page_types.remove(u'ftw.simplelayout.ContentPage')
    registry['plone.default_page_types'] = default_page_types

    contains_objects = registry['plone.contains_objects']
    contains_objects.remove(u'ftw.simplelayout.ContentPage')
    contains_objects.remove(u'ftw.simplelayout.FileListingBlock')
    contains_objects.remove(u'ftw.simplelayout.GalleryBlock')
    registry['plone.contains_objects'] = contains_objects

    displayed_types = list(registry['plone.displayed_types'])
    displayed_types.remove(u'ftw.simplelayout.ContentPage')
    registry['plone.displayed_types'] = tuple(displayed_types)



from ftw.upgrade import UpgradeStep


class MoveJavascriptFileOutOfClientDirectory(UpgradeStep):
    """Move javascript file out of client directory.
    """

    def __call__(self):
        self.install_upgrade_profile()



from collective.geo.mapwidget.browser.widget import MapWidget
from ftw.simplelayout.browser.blocks.base import BaseBlock
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class BlockMapWidget(MapWidget):

    def map_defaults(self):
        defaults = super(BlockMapWidget, self).map_defaults()

        zoom = getattr(self.context, 'zoomlevel', None)
        if zoom is not None:
            defaults['zoom'] = zoom

        maplayer = getattr(self.context, 'maplayer', None)
        if maplayer is not None:
            defaults['maplayer'] = maplayer

        return defaults


class MapBlockView(BaseBlock):

    template = ViewPageTemplateFile('templates/mapblock.pt')

    def get_address_map(self):
        address_map = BlockMapWidget(self, self.request, self.context)
        address_map.mapid = "geo-%s" % self.context.getId()
        address_map.addClass('block-map')
        address_map.klass = 'blockwidget-cgmap'

        return address_map



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.interfaces import IBlockConfiguration
from ftw.simplelayout.interfaces import IBlockModifier
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_CONTENT_TESTING
from ftw.simplelayout.utils import IS_PLONE_5
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import factoriesmenu
from ftw.testbrowser.pages import plone
from unittest import skipUnless
from unittest import TestCase
from zope.component import getMultiAdapter
import json


class TestSampleTypes(TestCase):

    layer = FTW_SIMPLELAYOUT_CONTENT_TESTING

    def setUp(self):
        super(TestSampleTypes, self).setUp()

        self.portal = self.layer['portal']

    @browsing
    def test_add_page(self, browser):
        browser.login().visit()
        factoriesmenu.add('ContentPage')
        browser.fill({'Title': u'This is a ContentPage'})
        browser.find_button_by_label('Save').click()
        self.assertEquals(u'This is a ContentPage', plone.first_heading())

    @browsing
    def test_add_textblock(self, browser):
        contentpage = create(Builder('sl content page').titled(u'A page'))

        browser.login().visit(contentpage)
        factoriesmenu.add('TextBlock')
        browser.fill({'Title': u'This is a ContentPage',
                      'Text': u'Some text'})
        browser.find_button_by_label('Save').click()
        browser.visit(contentpage)

        self.assertTrue(len(browser.css('.sl-block')), 'Expect one block')

    def test_textblock_modified(self):
        contentpage = create(Builder('sl content page').titled(u'A page'))
        block = create(Builder('sl textblock').within(contentpage))

        data = {'scale': 'mini'}
        modifier = getMultiAdapter((block, block.REQUEST), IBlockModifier)
        modifier.modify(data)

        conf = IBlockConfiguration(block).load()
        self.assertEquals('mini', conf.get('scale'))

    @browsing
    def test_contentpage_without_show_title_behavior(self, browser):
        page = create(Builder('sl content page')
                      .titled(u'A page')
                      .within(self.portal))

        browser.login().visit(page)

        self.assertEqual(
            ['A page'],
            browser.css('.documentFirstHeading').text)

    @browsing
    @skipUnless(IS_PLONE_5, 'Only testable on plone 5')
    def test_patterns_lib_tinymce_config_prependToUrl(self, browser):
        browser.login().visit(self.portal)
        factoriesmenu.add('TextBlock')
        tiny_settings = json.loads(
            browser.css('.richtext-field').first.attrib['data-pat-tinymce']
        )

        self.assertEquals('resolveuid/', tiny_settings['prependToUrl'])



from ftw.simplelayout.interfaces import ISimplelayoutBlock
from plone.app.content.browser.selection import DefaultPageSelectionView


def filter_simplelayout_blocks(brain):
    _catalog = brain.portal_catalog._catalog
    ifaces = _catalog.indexes['object_provides']._unindex.get(brain.getRID())
    return ISimplelayoutBlock.__identifier__ not in ifaces


class SimplelayoutDefaultPageSelectionView(DefaultPageSelectionView):

    def get_selectable_items(self):
        result = super(SimplelayoutDefaultPageSelectionView,
                       self).get_selectable_items()

        return filter(filter_simplelayout_blocks, result)



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.interfaces import IPageConfiguration
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING
from ftw.simplelayout.testing import SimplelayoutTestCase
from ftw.testbrowser import browsing
from plone.uuid.interfaces import IUUID
import transaction


class TestSimplelayoutRestApi(SimplelayoutTestCase):
    layer = FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING

    def setUp(self):
        super(TestSimplelayoutRestApi, self).setUp()
        self.layer['portal'].manage_permission(
            'plone.restapi: Use REST API',
            roles=['Anonymous']
        )
        self.setup_sample_ftis(self.layer['portal'])

    @browsing
    def test_page_state_is_in_response(self, browser):
        page = create(Builder('sample container'))

        config = {u'default': [{u'cols': [{u'blocks': [{u'uid': u'foo'}]}]},
                               {u'cols': [{u'blocks': [{u'uid': u'bar'}]},
                                          {u'blocks': []}]}],
                  u'sidebar': [{u'cols': [{u'blocks': [{u'uid': u'baz'},
                                                       {u'uid': u'foobar'}]}]}]}

        IPageConfiguration(page).store(config)
        transaction.commit()

        browser.open(page.absolute_url(), method='GET',
                     headers={'Accept': 'application/json'})
        self.assertDictEqual(config, browser.json['simplelayout'])

    @browsing
    def test_page_state_is_in_response_on_plone_root(self, browser):
        config = {u'default': [{u'cols': [{u'blocks': [{u'uid': u'foo'}]}]},
                               {u'cols': [{u'blocks': [{u'uid': u'bar'}]},
                                          {u'blocks': []}]}],
                  u'sidebar': [{u'cols': [{u'blocks': [{u'uid': u'baz'},
                                                       {u'uid': u'foobar'}]}]}]}

        IPageConfiguration(self.layer['portal']).store(config)
        transaction.commit()

        browser.open(self.layer['portal'].absolute_url(), method='GET',
                     headers={'Accept': 'application/json'})
        self.assertDictEqual(config, browser.json['simplelayout'])

    @browsing
    def test_blocks_are_resturned_as_uid_map(self, browser):

        page = create(Builder('sample container'))
        block1 = create(Builder('sample block').within(page))
        block2 = create(Builder('sample block').within(page))

        IPageConfiguration(page).store(
            {'default': [{'cols': [{'blocks': [
                {'uid': IUUID(block1)}, {'uid': IUUID(block2)}
            ]}]}]})
        transaction.commit()

        browser.open(page.absolute_url(), method='GET',
                     headers={'Accept': 'application/json'})

        self.assertItemsEqual(
            [IUUID(block1), IUUID(block2)],
            browser.json['slblocks'].keys()
        )

        self.assertEquals(
            'SampleBlock',
            browser.json['slblocks'][IUUID(block1)]['@type']
        )
        self.assertEquals(
            'SampleBlock',
            browser.json['slblocks'][IUUID(block2)]['@type']
        )



from ftw.simplelayout.interfaces import IBlockProperties
from ftw.simplelayout.interfaces import ISimplelayoutBlock
from pkg_resources import get_distribution
from plone import api
from plone.dexterity.interfaces import IDexterityFTI
from plone.dexterity.utils import safe_utf8
from plone.i18n.normalizer.interfaces import IIDNormalizer
from Products.CMFCore.utils import getToolByName
from zope.component import getUtility
from zope.component import queryMultiAdapter
from zope.component.hooks import getSite
import pkg_resources


try:
    pkg_resources.get_distribution('ftw.trash')
except pkg_resources.DistributionNotFound:
    FTW_TRASH_SUPPORT = False
else:
    FTW_TRASH_SUPPORT = True
    from ftw.trash.interfaces import ITrashed


IS_PLONE_5 = get_distribution('Plone').version >= '5'


def normalize_portal_type(portal_type):
    normalizer = getUtility(IIDNormalizer)
    return normalizer.normalize(portal_type)


def get_block_html(block):
    properties = queryMultiAdapter((block, block.REQUEST),
                                   IBlockProperties)
    viewname = properties.get_current_view_name()
    return block.restrictedTraverse(safe_utf8(viewname))()


def get_block_types():
    platform = getSite()
    types_tool = getToolByName(platform, 'portal_types')

    dexterity_ftis = filter(
        IDexterityFTI.providedBy, types_tool.objectValues())

    return filter(
        lambda fti: ISimplelayoutBlock.__identifier__ in fti.behaviors,
        dexterity_ftis)


def unrestricted_uuidToObject(uuid):
    platform = getSite()
    catalog = getToolByName(platform, 'portal_catalog', None)
    if catalog is None:
        return None

    result = catalog.unrestrictedSearchResults(UID=uuid)
    if len(result) != 1:
        return None

    with api.env.adopt_roles(roles=['Manager']):
        return result[0].getObject()


def is_trashed(obj):
    if FTW_TRASH_SUPPORT:
        return ITrashed.providedBy(obj)
    return False



from ftw.simplelayout.images.configuration import Configuration
from ftw.simplelayout.interfaces import ISimplelayoutDefaultSettings
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_CONTENT_TESTING
from ftw.simplelayout.testing import SimplelayoutTestCase
from ftw.testbrowser import browsing
from plone import api
import transaction


class TestImageConfigurationImageLimits(SimplelayoutTestCase):

    layer = FTW_SIMPLELAYOUT_CONTENT_TESTING

    def _set_image_limit(self, value):
        api.portal.set_registry_record(
            name='image_limits',
            value=value,
            interface=ISimplelayoutDefaultSettings)

        transaction.commit()

    def test_format_with_no_configuration(self):
        self._set_image_limit({})
        self.assertEqual({}, Configuration().image_limits())

    @browsing
    def test_missing_value(self, browser):
        self._set_image_limit({})

        browser.login().visit(self.layer['portal'],
                              view='simplelayout-controlpanel')

        browser.find_button_by_label('Save').click()
        self.assertEqual({}, Configuration().image_limits())

    def test_format_with_only_one_limit_type(self):
        self._set_image_limit({
            'example.contenttype_1': [u'soft: width=400']
            })

        self.assertDictEqual(
            {'example.contenttype_1': {
                'soft': {
                    'width': 400,
                }
            }},
            Configuration().image_limits())

    def test_format_with_soft_and_hard_limits(self):
        self._set_image_limit({
            'example.contenttype_1': [
                u'soft: width=400, height=200',
                u'hard: width=300',
                ]
            })

        self.assertDictEqual(
            {'example.contenttype_1': {
                'soft': {
                    'width': 400,
                    'height': 200,
                },
                'hard': {
                    'width': 300,
                }
            }},
            Configuration().image_limits())

    def test_format_with_multiple_contenttypes(self):
        self._set_image_limit({
            'example.contenttype_1': [
                u'soft: width=400, height=200',
                u'hard: width=300',
                ],
            'example.contenttype_2': [
                u'soft: width=800',
                ]
            })

        self.assertDictEqual(
            {
                'example.contenttype_1': {
                    'soft': {
                        'width': 400,
                        'height': 200,
                    },
                    'hard': {
                        'width': 300,
                    }
                },
                'example.contenttype_2': {
                    'soft': {
                        'width': 800,
                    }
                }
            },
            Configuration().image_limits())


class TestImageConfigurationIAspectRatios(SimplelayoutTestCase):

    layer = FTW_SIMPLELAYOUT_CONTENT_TESTING

    def _set_aspect_rations(self, value):
        api.portal.set_registry_record(
            name='image_cropping_aspect_ratios',
            value=value,
            interface=ISimplelayoutDefaultSettings)

        transaction.commit()

    def test_format_with_no_configuration(self):
        self._set_aspect_rations({})
        self.assertEqual({}, Configuration().aspect_ratios())

    @browsing
    def test_missing_value(self, browser):
        self._set_aspect_rations({})

        browser.login().visit(self.layer['portal'],
                              view='simplelayout-controlpanel')

        browser.find_button_by_label('Save').click()
        self.assertEqual({}, Configuration().image_limits())

    def test_format_with_aspect_ratios(self):
        self._set_aspect_rations({
            'example.contenttype_1': [u'4/3 => 1.33333', u'16/9 => 1.7777']
            })

        self.assertDictEqual({
            'example.contenttype_1': [
                {'title': '4/3', 'value': '1.33333'},
                {'title': '16/9', 'value': '1.7777'},
            ]}, Configuration().aspect_ratios())

    def test_format_with_multiple_contenttypes(self):
        self._set_aspect_rations({
            'example.contenttype_1': [u'4/3 => 1.33333', u'16/9 => 1.7777'],
            'example.contenttype_2': [u'Free => 0']
            })

        self.assertDictEqual({
            'example.contenttype_1': [
                {'title': '4/3', 'value': '1.33333'},
                {'title': '16/9', 'value': '1.7777'},
            ],
            'example.contenttype_2': [
                {'title': 'Free', 'value': '0'},
            ]}, Configuration().aspect_ratios())



# pylint: disable=E0211, E0213
# E0211: Method has no argument
# E0213: Method should have "self" as first argument

from ftw.simplelayout.interfaces import ISimplelayout
from zope.interface import Interface


class ITextBlock(Interface):
    """Marker interface for TextBlocks"""


class IVideoBlock(Interface):
    """Marker interface for VideoBlocks"""


class IFileListingBlock(Interface):
    """Marker interface for TextBlocks"""


class IListingBlockColumns(Interface):
    """Marker interface for TextBlocks"""

    def columns():
        """Returns an ftw.table compatible list of columns"""


class IGalleryBlock(Interface):
    """Marker interface for GalleryBlock"""


class IContentPage(ISimplelayout):
    """Marker interface for ConetPages"""



from ftw.simplelayout.interfaces import IBlockModifier
from ftw.simplelayout.interfaces import IBlockProperties
from plone.app.uuid.utils import uuidToObject
from zExceptions import BadRequest
from zope.component import getMultiAdapter
from zope.component import queryMultiAdapter
from zope.publisher.browser import BrowserView
import json


class ReloadBlockView(BrowserView):
    """Reloads a block and performs several modifications before.
    By default it tries to change the view, if the view_name parameter is
    present in the request. Afterwards the BlockReloader adapter gets called,
    which performs block specific modification, like storing image scales.
    Finally the new block html will be returned."""

    def __init__(self, context, request):
        super(ReloadBlockView, self).__init__(context, request)
        self.data = None
        self.block = None
        self.properties = None

    def __call__(self):
        self.data = json.loads(self.request.get('data', "{}"))
        self._get_block()
        self._set_new_view()
        self._block_specific_modifications()
        return self._render_block()

    def _get_block(self):
        uid = self.data.get('uid', None)
        if uid is None:
            raise BadRequest('No uid provided.')

        self.block = uuidToObject(uid)
        self.properties = getMultiAdapter((self.block, self.block.REQUEST),
                                          IBlockProperties)

    def _render_block(self):
        return self.block.restrictedTraverse(
            self.properties.get_current_view_name())()

    def _block_specific_modifications(self):
        modifier = queryMultiAdapter((self.block, self.block.REQUEST),
                                     IBlockModifier)
        if modifier is not None:
            modifier.modify(self.data)

    def _set_new_view(self):
        view_name = self.data.get('view_name', None)
        if view_name is None:
            return

        try:
            self.properties.set_view(view_name.encode('utf-8'))

        except ValueError:
            raise BadRequest(
                'The view "%s" is not available on this block.' % (
                    view_name))

        except RuntimeError:
            raise BadRequest(
                'This object does not support changing the view.')



from collective.dexteritytextindexer import indexer
from ftw.simplelayout.interfaces import ISimplelayout
from ftw.simplelayout.interfaces import ISimplelayoutBlock
from ftw.simplelayout.utils import is_trashed
from zope.component import adapts


class BlockSearchableTextIndexer(object):
    adapts(ISimplelayout)

    def __init__(self, context):
        self.context = context

    def __call__(self):
        searchable_text = ''
        for content in self.context.objectValues():
            if ISimplelayoutBlock.providedBy(content) and not is_trashed(content):
                searchable_text += indexer.dynamic_searchable_text_indexer(
                    content)()
        return searchable_text



from datetime import datetime
from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.configuration import synchronize_page_config_with_blocks
from ftw.simplelayout.contenttypes.contents.interfaces import IContentPage
from ftw.simplelayout.interfaces import IBlockConfiguration
from ftw.simplelayout.interfaces import IPageConfiguration
from ftw.simplelayout.staging.interfaces import IBaseline
from ftw.simplelayout.staging.interfaces import IStaging
from ftw.simplelayout.staging.interfaces import IWorkingCopy
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_CONTENT_TESTING
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import statusmessages
from ftw.testing import freeze
from ftw.testing import staticuid
from plone.app.textfield.value import RichTextValue
from plone.uuid.interfaces import IUUID
from unittest import TestCase
from zope.component import adapter
from zope.interface.verify import verifyObject
from zope.lifecycleevent.interfaces import IObjectAddedEvent
import transaction


@adapter(IContentPage, IObjectAddedEvent)
def create_block_when_page_is_added(parent, event):
    create(Builder('sl textblock').titled(u'Auto generated').within(parent))


class TestWorkingCopy(TestCase):
    layer = FTW_SIMPLELAYOUT_CONTENT_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

        for fti in (self.portal.portal_types.File,
                    self.portal.portal_types.Image):
            if not hasattr(fti, 'behaviors'):
                # File and image are not DX in Plone 4.3.x, but AT.
                continue
            behaviors = list(fti.behaviors)
            behaviors.remove('plone.app.dexterity.behaviors.filename.INameFromFileName')
            behaviors += ['plone.app.content.interfaces.INameFromTitle']
            fti.behaviors = tuple(behaviors)

    def test_staging_manager_implements_interface(self):
        page = create(Builder('sl content page'))
        verifyObject(IStaging, IStaging(page))

    def test_is_baseline(self):
        baseline = create(Builder('sl content page').titled(u'A page'))
        self.assertFalse(IStaging(baseline).is_baseline())
        working_copy = IStaging(baseline).create_working_copy(self.portal)
        self.assertTrue(IStaging(baseline).is_baseline())
        self.assertFalse(IStaging(working_copy).is_baseline())

    def test_is_working_copy(self):
        baseline = create(Builder('sl content page').titled(u'A page'))
        self.assertFalse(IStaging(baseline).is_working_copy())
        working_copy = IStaging(baseline).create_working_copy(self.portal)
        self.assertFalse(IStaging(baseline).is_working_copy())
        self.assertTrue(IStaging(working_copy).is_working_copy())

    def test_get_baseline(self):
        baseline = create(Builder('sl content page').titled(u'A page'))
        self.assertIsNone(IStaging(baseline).get_baseline())
        working_copy = IStaging(baseline).create_working_copy(self.portal)
        self.assertIsNone(IStaging(baseline).get_baseline())
        self.assertEquals(baseline, IStaging(working_copy).get_baseline())

    def test_get_working_copies(self):
        baseline = create(Builder('sl content page').titled(u'A page'))
        self.assertIsNone(IStaging(baseline).get_working_copies())
        working_copy = IStaging(baseline).create_working_copy(self.portal)
        self.assertEquals([working_copy], IStaging(baseline).get_working_copies())
        self.assertIsNone(IStaging(working_copy).get_working_copies())

    def test_create_working_copy_of_page(self):
        baseline = create(Builder('sl content page').titled(u'A page'))
        self.assert_staging_interfaces((), baseline)
        working_copy = IStaging(baseline).create_working_copy(self.portal)
        self.assertTrue(baseline._p_oid)
        self.assertTrue(working_copy._p_oid)
        self.assertNotEquals(baseline._p_oid, working_copy._p_oid)
        self.assertNotEquals(IUUID(baseline), IUUID(working_copy))
        self.assertEquals(baseline.Title(), working_copy.Title())
        self.assert_staging_interfaces({IBaseline}, baseline)
        self.assert_staging_interfaces({IWorkingCopy}, working_copy)

    def test_working_copy_contains_blocks_but_not_child_pages(self):
        baseline = create(Builder('sl content page').titled(u'A page'))
        create(Builder('sl content page').titled(u'Child page').within(baseline))
        create(Builder('sl textblock').titled(u'Textblock').within(baseline))
        self.assertEquals({'child-page', 'textblock'}, set(baseline.objectIds()))
        self.assertEquals(2, baseline.objectCount())

        working_copy = IStaging(baseline).create_working_copy(self.portal)
        self.assertEquals({'textblock'}, set(working_copy.objectIds()))
        self.assertEquals(1, working_copy.objectCount())
        self.assertEquals({'child-page', 'textblock'}, set(baseline.objectIds()))
        self.assertEquals(2, baseline.objectCount())
        self.assertNotEquals(IUUID(baseline['textblock']), IUUID(working_copy['textblock']))

    def test_UIDS_in_sl_state_are_updated_when_creating_working_copy(self):
        with staticuid('baseline'):
            baseline = create(Builder('sl content page').titled(u'A page')
                              .with_blocks(Builder('sl textblock').titled(u'First'),
                                           Builder('sl textblock').titled(u'Second')))

        self.assertEquals(
            {'default': [{'cols': [{'blocks': [
                {'uid': 'baseline000000000000000000000002'},
                {'uid': 'baseline000000000000000000000003'}]}]}]},
            IPageConfiguration(baseline).load())

        with staticuid('workingcopy'):
            working_copy = IStaging(baseline).create_working_copy(self.portal)

        self.assertEquals(
            {'default': [{'cols': [{'blocks': [
                {'uid': 'workingcopy000000000000000000001'},
                {'uid': 'workingcopy000000000000000000002'}]}]}]},
            IPageConfiguration(working_copy).load())

    def test_apply_working_copy_copies_field_values_back_to_baseline(self):
        bl_page = create(Builder('sl content page')
                         .titled(u'original page title')
                         .with_blocks(Builder('sl textblock')
                                      .titled(u'original block title')
                                      .having(text=RichTextValue(
                                          u'<p>original block text</p>'))))

        wc_page = IStaging(bl_page).create_working_copy(self.portal)
        wc_page.setTitle(u'modified page title')
        wc_block = wc_page.objectValues()[0]
        wc_block.setTitle(u'modified block title')
        wc_block.text = RichTextValue(u'<p>modified block text</p>')

        IStaging(wc_page).apply_working_copy()
        bl_block = bl_page.objectValues()[0]
        self.assertEquals('modified page title', wc_page.Title())
        self.assertEquals('modified block title', bl_block.Title())
        self.assertEquals('<p>modified block text</p>', bl_block.text.output)

    def test_applying_removes_blocks_from_baseline_when_removed_in_working_copy(self):
        bl_page = create(Builder('sl content page').titled(u'Page')
                         .with_blocks(Builder('sl textblock').titled(u'Foo'))
                         .with_blocks(Builder('sl textblock').titled(u'Bar')))
        create(Builder('sl content page').titled(u'Subpage').within(bl_page))
        self.assertEquals(['foo', 'bar', 'subpage'], bl_page.objectIds())

        wc_page = IStaging(bl_page).create_working_copy(self.portal)
        self.assertEquals(['foo', 'bar'], wc_page.objectIds())
        wc_page.manage_delObjects(['foo'])
        self.assertEquals(['bar'], wc_page.objectIds())

        IStaging(wc_page).apply_working_copy()
        self.assertEquals(['bar', 'subpage'], bl_page.objectIds())

    def test_applying_adds_new_blocks(self):
        bl_page = create(Builder('sl content page').titled(u'Page'))
        self.assertEquals([], bl_page.objectIds())

        wc_page = IStaging(bl_page).create_working_copy(self.portal)
        create(Builder('sl textblock').titled(u'Foo').within(wc_page))
        IStaging(wc_page).apply_working_copy()

        self.assertEquals(['foo'], bl_page.objectIds())

    def test_applying_adds_new_containers_and_handles_recursion(self):
        bl_page = create(Builder('sl content page').titled(u'Page'))
        self.assertEquals([], bl_page.objectIds())

        wc_page = IStaging(bl_page).create_working_copy(self.portal)
        gallery = create(Builder('sl galleryblock').titled(u'Pictures').within(wc_page))
        create(Builder('image').titled('Sea').within(gallery))
        IStaging(wc_page).apply_working_copy()

        self.assertEquals(['pictures'], bl_page.objectIds())
        self.assertEquals(['sea'], bl_page.pictures.objectIds())

    def test_files_in_filelistingblocks_are_synced(self):
        bl_page = create(
            Builder('sl content page').titled(u'Page')
            .with_blocks(Builder('sl listingblock').titled(u'Downloads')
                         .with_files(Builder('file').titled(u'One'),
                                     Builder('file').titled(u'Two'))))

        wc_page = IStaging(bl_page).create_working_copy(self.portal)
        wc_page.downloads.one.setTitle('The first one')
        create(Builder('file').titled(u'Three').within(wc_page.downloads))
        wc_page.downloads.manage_delObjects(['two'])
        IStaging(wc_page).apply_working_copy()

        self.assertEquals('The first one', bl_page.downloads.one.Title())
        self.assertEquals(['one', 'three'], bl_page.downloads.objectIds())

    @browsing
    def test_internal_links_to_files_in_filelistingblocks_are_synced(self, browser):
        bl_page = create(Builder('sl content page').titled(u'Page'))
        wc_page = IStaging(bl_page).create_working_copy(self.portal)

        listingblock = create(Builder('sl listingblock').titled(u'Downloads').within(wc_page))
        file1 = create(Builder('file').titled(u'One').within(listingblock))

        create(Builder('sl textblock').titled(u'Link on file').within(wc_page)
               .having(text=RichTextValue(u'''
<p>
  <a class="internal-link" href="resolveuid/{0}">One</a>
</p>
                            '''.format(IUUID(file1)).strip())))

        IStaging(wc_page).apply_working_copy()

        transaction.commit()
        self.assertEquals(
            'resolveuid/{}'.format(IUUID(self.portal.page.downloads.one)),
            browser.login().visit(self.portal.page).css('a.internal-link').first.attrib['href'])

    def test_childrens_children_order_is_preserved(self):
        bl_page = create(Builder('sl content page').titled(u'Page'))
        bl_dow = create(Builder('sl listingblock').titled(u'Downloads')
                        .within(bl_page)
                        .with_files(Builder('file').titled(u'One'),
                                    Builder('file').titled(u'Two')))
        create(Builder('sl content page').titled(u'Sub Page 1').within(bl_page))
        create(Builder('sl content page').titled(u'Sub Page 2').within(bl_page))

        wc_page = IStaging(bl_page).create_working_copy(self.portal)

        self.assertEquals(['one', 'two'], bl_dow.objectIds())
        self.assertEquals(['downloads', 'sub-page-1', 'sub-page-2'], bl_page.objectIds())

        IStaging(wc_page).apply_working_copy()
        self.assertEquals(['one', 'two'], bl_dow.objectIds())
        self.assertEquals(['downloads', 'sub-page-1', 'sub-page-2'], bl_page.objectIds())

        wc_page = IStaging(bl_page).create_working_copy(self.portal)
        wc_page.downloads.moveObjectsToTop('two')
        IStaging(wc_page).apply_working_copy()
        self.assertEquals(['two', 'one'], bl_dow.objectIds())

        wc_page = IStaging(bl_page).create_working_copy(self.portal)
        create(Builder('file').titled(u'Three').within(wc_page.downloads))
        wc_page.downloads.moveObjectsUp('three')
        IStaging(wc_page).apply_working_copy()
        self.assertEquals(['two', 'three', 'one'], bl_dow.objectIds())

    def test_sl_page_state_is_updated_when_applying_working_copy(self):
        with staticuid('baseline'):
            baseline = create(Builder('sl content page').titled(u'A page')
                              .with_blocks(Builder('sl textblock').titled(u'First')))

        self.assertEquals(
            {'default': [{'cols': [{'blocks': [
                {'uid': 'baseline000000000000000000000002'}]}]}]},
            IPageConfiguration(baseline).load())

        with staticuid('workingcopy'):
            working_copy = IStaging(baseline).create_working_copy(self.portal)

        self.assertEquals(
            {'default': [{'cols': [{'blocks': [
                {'uid': 'workingcopy000000000000000000001'}]}]}]},
            IPageConfiguration(working_copy).load())

        with staticuid('editing'):
            create(Builder('sl textblock').titled(u'Second').within(working_copy))
            synchronize_page_config_with_blocks(working_copy)

        self.assertEquals(
            {'default': [{'cols': [{'blocks': [
                {'uid': 'workingcopy000000000000000000001'},
                {'uid': 'editing0000000000000000000000001'}]}]}]},
            IPageConfiguration(working_copy).load())

        with staticuid('applying'):
            IStaging(working_copy).apply_working_copy()

        self.assertEquals(
            {'default': [{'cols': [{'blocks': [
                {'uid': 'baseline000000000000000000000002'},
                {'uid': 'editing0000000000000000000000001'}]}]}]},
            IPageConfiguration(baseline).load())

    def test_sl_block_state_is_copied_when_applying(self):
        baseline = create(Builder('sl content page').titled(u'A page')
                          .with_blocks(Builder('sl textblock').titled(u'Block')))
        self.assertEqual({}, IBlockConfiguration(baseline.block).load())

        working_copy = IStaging(baseline).create_working_copy(self.portal)
        self.assertEqual({}, IBlockConfiguration(working_copy.block).load())

        IBlockConfiguration(working_copy.block).store({'scale': 'mini'})
        self.assertEqual({}, IBlockConfiguration(baseline.block).load())
        self.assertEqual({'scale': 'mini'}, IBlockConfiguration(working_copy.block).load())

        IStaging(working_copy).apply_working_copy()
        self.assertEqual({'scale': 'mini'}, IBlockConfiguration(baseline.block).load())

    def test_html_references_within_page_are_rewritten(self):
        self.maxDiff = None
        bl_page = create(Builder('sl content page').titled(u'A page'))
        with staticuid('baseline'):
            bl_one = create(Builder('sl textblock').titled(u'One').within(bl_page))
            self.assertEquals('baseline000000000000000000000001', IUUID(bl_one))
            bl_two = create(Builder('sl textblock').titled(u'Two').within(bl_page))
            self.assertEquals('baseline000000000000000000000002', IUUID(bl_two))
            create(Builder('sl textblock').titled(u'ToC').within(bl_page)
                   .having(text=RichTextValue(u'''
<p>
  <a class="internal-link" href="resolveuid/baseline000000000000000000000001">One</a>
</p>
                            '''.strip())))

        with staticuid('workingcopy'):
            wc_page = IStaging(bl_page).create_working_copy(self.portal)
            self.assertEquals('workingcopy000000000000000000001', IUUID(wc_page['one']))
            self.assertEquals('workingcopy000000000000000000002', IUUID(wc_page['two']))

        self.assertMultiLineEqual(u'''
<p>
  <a class="internal-link" href="resolveuid/workingcopy000000000000000000001">One</a>
</p>
        '''.strip(), wc_page.toc.text.raw.strip())

        with staticuid('editing'):
            wc_three = create(Builder('sl textblock').titled(u'Three').within(wc_page))
            self.assertEquals('editing0000000000000000000000001', IUUID(wc_three))

        wc_page.toc.text = RichTextValue(u'''
<p>
  <a class="internal-link" href="resolveuid/workingcopy000000000000000000001">One</a>
  <a class="internal-link" href="resolveuid/workingcopy000000000000000000002">Two</a>
  <a class="internal-link" href="resolveuid/editing0000000000000000000000001">Three</a>
</p>
        '''.strip())

        with staticuid('apply'):
            IStaging(wc_page).apply_working_copy()

        self.assertMultiLineEqual(u'''
<p>
  <a class="internal-link" href="resolveuid/baseline000000000000000000000001">One</a>
  <a class="internal-link" href="resolveuid/baseline000000000000000000000002">Two</a>
  <a class="internal-link" href="resolveuid/editing0000000000000000000000001">Three</a>
</p>
        '''.strip(), bl_page.toc.text.raw.strip())

    def test_working_copy_is_removed_after_applying(self):
        bl_page = create(Builder('sl content page').titled(u'A page'))
        wc_page = IStaging(bl_page).create_working_copy(self.portal)
        self.assertIn(wc_page.getId(), self.portal.objectIds())
        IStaging(wc_page).apply_working_copy()
        self.assertNotIn(wc_page.getId(), self.portal.objectIds())

    def test_discard_working_copy(self):
        baseline = create(Builder('sl content page').titled(u'A page'))
        self.assertFalse(IStaging(baseline).is_baseline())
        self.assertIsNone(IStaging(baseline).get_working_copies())
        self.assertFalse(IStaging(baseline).is_working_copy())
        self.assertIsNone(IStaging(baseline).get_baseline())

        working_copy = IStaging(baseline).create_working_copy(self.portal)
        self.assertTrue(IStaging(baseline).is_baseline())
        self.assertEquals([working_copy], IStaging(baseline).get_working_copies())
        self.assertTrue(IStaging(working_copy).is_working_copy())
        self.assertEquals(baseline, IStaging(working_copy).get_baseline())

        IStaging(working_copy).discard_working_copy()
        self.assertNotIn(working_copy.getId(), self.portal.objectIds())
        self.assertFalse(IStaging(baseline).is_baseline())
        self.assertIsNone(IStaging(baseline).get_working_copies())
        self.assertFalse(IStaging(baseline).is_working_copy())
        self.assertIsNone(IStaging(baseline).get_baseline())

    @browsing
    def test_message_is_displayed_on_working_copy(self, browser):
        baseline = create(Builder('sl content page').titled(u'A page'))
        with freeze(datetime(2017, 7, 24)):
            working_copy = IStaging(baseline).create_working_copy(self.portal)

        transaction.commit()

        browser.login().open(working_copy)
        statusmessages.assert_message(
            'This is the working copy of this baseline, created by '
            'test_user_1_ at Jul 24, 2017.')

    @browsing
    def test_message_is_displayed_on_baseline(self, browser):
        baseline = create(Builder('sl content page').titled(u'A page'))
        with freeze(datetime(2017, 7, 24)):
            IStaging(baseline).create_working_copy(self.portal)

        transaction.commit()

        browser.login().open(baseline)
        statusmessages.assert_message(
            'test_user_1_ is working on this page in a working copy created '
            'at Jul 24, 2017.')

    @browsing
    def test_does_not_break_with_reference_to_sub_page(self, browser):
        """When pasting an object, the linkintegrity checker verifies the references.
        When creating the working copy, the tree is filtered while copy/pasting the
        page.
        Regression: when a block of the baseline contains a link to a sub page, the link
        integrity checker failed since it was executed while the tree still was filtered
        and thus the sub page was not reachable with traversal at all.
        """
        baseline = create(Builder('sl content page').titled(u'Baseline'))

        with staticuid('childpage'):
            create(Builder('sl content page').titled(u'Childpage')
                   .within(baseline))

        create(Builder('sl textblock').titled(u'Block').within(baseline)
               .having(text=RichTextValue(u'''
               <p>
                   <a class="internal-link"
                      href="resolveuid/childpage00000000000000000000001">Childpage</a>
               </p>'''.strip())))

        IStaging(baseline).create_working_copy(self.portal)

    @browsing
    def test_no_children_added_when_cloning(self, browser):
        """Some FTIs have subscribors which add children when an object is created.
        Example: when a news folder (ftw.news) is created, a news listing block is added.

        But when creating a working copy of such an object, we probably already have this
        kind of block and do not want to create an additional one in the clone step.

        Therefore all children created in the cloning step must be removed.
        """
        self.portal.getSiteManager().registerHandler(create_block_when_page_is_added)

        baseline = create(Builder('sl content page').titled(u'Baseline'))
        self.assertEquals(['auto-generated'], baseline.objectIds())

        working_copy = IStaging(baseline).create_working_copy(self.portal)
        self.assertEquals(['auto-generated'], working_copy.objectIds())

    def assert_staging_interfaces(self, expected, obj):
        expected = set(expected)
        got = set(filter(lambda iface: iface.providedBy(obj), (IBaseline, IWorkingCopy)))
        self.assertEquals(expected, got, 'Provided interfaces unexpected for {!r}'.format(obj))



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING
from ftw.simplelayout.testing import SimplelayoutTestCase
from ftw.testbrowser import browsing


class TestRemoveBlockPloneUI(SimplelayoutTestCase):

    layer = FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.setup_sample_ftis(self.portal)
        self.setup_block_views()

    @browsing
    def test_parent_modified_date_has_changed(self, browser):
        page = create(Builder('sample container'))
        block = create(Builder('sample block').within(page))

        cached_date = page.modification_date

        browser.login().visit(block, view='edit')
        browser.find_button_by_label('Save').click()
        self.assertNotEquals(cached_date,
                             page.modification_date,
                             'Modification date of sl container should change')

    @browsing
    def test_modification_date_of_plone_root_has_changed(self, browser):
        block = create(Builder('sample block').within(self.portal))
        cached_date = self.portal.modified()

        browser.login().visit(block, view='edit')
        browser.find_button_by_label('Save').click()

        self.assertNotEquals(cached_date,
                             self.portal.modified(),
                             'Modification date of portal should change')



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING
from ftw.simplelayout.testing import SimplelayoutTestCase
from ftw.testbrowser import browsing


class RedirectToParentView(SimplelayoutTestCase):

    layer = FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING

    def setUp(self):
        self.setup_sample_ftis(self.layer['portal'])
        self.container = create(Builder('sample container'))
        self.block = create(Builder('sample block').within(self.container))

    @browsing
    def test_view_is_redirecting_to_parent_with_anchor(self, browser):
        browser.login().visit(self.block, view='@@redirect_to_parent')
        self.assertEquals(self.container.absolute_url() + '#' + self.block.id,
                          browser.url)

    @browsing
    def test_redirect_to_parent_if_anonymous(self, browser):
        browser.visit(self.container, view='@@redirect_to_parent')
        self.assertEquals(
            self.layer['portal'].absolute_url() + '#' + self.container.id,
            browser.url)

    @browsing
    def test_do_NOT_redirect_to_parent_if_NOT_anonymous(self, browser):
        browser.login().visit(self.container, view='@@redirect_to_parent')
        self.assertEquals(self.container.absolute_url() + '/folder_contents',
                          browser.url)



from collections import OrderedDict
from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.browser.actions import DefaultActions
from ftw.simplelayout.interfaces import ISimplelayoutActions
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_INTEGRATION_TESTING
from ftw.simplelayout.testing import SimplelayoutTestCase
from zope.component import getMultiAdapter
from zope.component import provideAdapter
from zope.interface import Interface


class TestBlockReloadView(SimplelayoutTestCase):

    layer = FTW_SIMPLELAYOUT_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.setup_sample_ftis(self.portal)
        self.container = create(Builder('sample container'))

    def test_amount_of_default_actions(self):
        actions = getMultiAdapter((self.container, self.container.REQUEST),
                                  ISimplelayoutActions)

        self.assertEquals(3, len(actions.default_actions()))

    def test_default_actions_are_move_edit_and_delete(self):
        actions = getMultiAdapter((self.container, self.container.REQUEST),
                                  ISimplelayoutActions)

        default_actions = actions.default_actions()

        self.assertIn('move', default_actions)
        self.assertIn('edit', default_actions)
        self.assertIn('delete', default_actions)

    def test_specific_actions_is_a_empty_ordered_dict_by_default(self):
        actions = getMultiAdapter((self.container, self.container.REQUEST),
                                  ISimplelayoutActions)

        self.assertTrue(isinstance(actions.specific_actions(), OrderedDict))
        self.assertFalse(len(actions.specific_actions()))

    def test_specific_actions_for_dummy_block(self):

        class DummyBlockActions(DefaultActions):

            def specific_actions(self):
                return OrderedDict(dummy={'class': '',
                                          'title': '',
                                          'href': '',
                                          'data-scale': 'mini'})

        provideAdapter(DummyBlockActions,
                       adapts=(Interface, Interface),
                       name='sample-actions')

        actions = getMultiAdapter((self.container, self.container.REQUEST),
                                  ISimplelayoutActions,
                                  name='sample-actions')

        self.assertEquals(4, len(actions.actions))
        self.assertIn('dummy', actions.actions)



from ftw.simplelayout.interfaces import ISimplelayoutDefaultSettings
from plone import api


class Configuration(object):
    """The image-related configuration requires a special format to be customizable.
    """
    def image_limits(self):
        # Example limit-configuration line:
        # {'ftw.simplelayout.TextBlock': [
        #     soft:width=123, height=222,
        #     hard:width=123]
        # }

        limits = {}
        entries = self._get_registry_property('image_limits')

        for portal_type, values in entries.items():
            limits[portal_type] = {}

            # [soft:width=123,height=222, hard:width=123]
            for value in values:
                value = value.encode('utf-8')

                # 'soft', ['width=123 ,height=222']
                limit_type, dimensions = map(str.strip, value.split(':'))
                limits[portal_type][limit_type] = {}

                # ['width=123', 'height=222']
                for dimension in map(str.strip, dimensions.split(',')):

                    # 'width', 123
                    dimension_name, value = map(str.strip, dimension.split('='))
                    limits[portal_type][limit_type][dimension_name] = int(value)

        # Output example
        # {'ftw.simplelayout.TextBlock':
        #     {'hard': {'width': '150'},
        #      'soft': {'height': '300', 'width': '400'}},
        #  'ftw.slider.Pane': {
        #      'hard': {'width': '150'},
        #      'soft': {'height': '300', 'width': '900'}}}
        return limits

    def aspect_ratios(self):
        # Example aspect_ration-configuration line:
        # {'ftw.simplelayout.TextBlock': ['4/3 => 1.33333', '16/9 => 0.7777']
        entries = self._get_registry_property('image_cropping_aspect_ratios')
        aspect_ratios = {}

        for portal_type, values in entries.items():
            aspect_ratios[portal_type] = []

            # ['4/3 => 1.33333', '16/9 => 0.7777']
            for value in values:
                value = value.encode('utf-8')

                # '4/3', '1.33333'
                title, ratio = map(str.strip, value.split('=>'))
                aspect_ratios[portal_type].append({
                    'title': title,
                    'value': ratio
                    })

        # Output example
        # {'ftw.simplelayout.TextBlock':
        #     [
        #         {"title": '4/3', "value": 1.33333},
        #         {"title": '16/9', "value": 1.77778}
        #     ]
        # }
        return aspect_ratios

    def _get_registry_property(self, name):
        return api.portal.get_registry_record(
            name=name, interface=ISimplelayoutDefaultSettings)



from ftw.simplelayout.staging.interfaces import IStaging
from plone import api
from plone.app.layout.viewlets import common
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class WorkingCopyViewlet(common.PathBarViewlet):
    index = ViewPageTemplateFile('templates/workingcopy_viewlet.pt')

    def render(self):
        return self.index()

    @property
    def owner_name(self):
        owner = self.context.getOwner()
        return owner.getProperty('fullname') or owner.getId()

    @property
    def baseline_url(self):
        baseline = IStaging(self.context).get_baseline()
        return baseline.absolute_url()


class BaselineViewlet(common.PathBarViewlet):
    index = ViewPageTemplateFile('templates/baseline_viewlet.pt')

    def render(self):
        if api.user.is_anonymous():
            return ''
        return self.index()

    def get_working_copies(self):
        return IStaging(self.context).get_working_copies()

    def owner_name(self, working_copy):
        owner = working_copy.getOwner()
        return owner.getProperty('fullname') or owner.getId()



from ftw.simplelayout.handlers import unwrap_persistence
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING
from ftw.simplelayout.testing import SimplelayoutTestCase
from persistent.list import PersistentList
from persistent.mapping import PersistentMapping


class TestUnwrapPersistance(SimplelayoutTestCase):

    layer = FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING

    def test_unwrap_persistance(self):
        config = PersistentMapping(
            {'default': PersistentList([
                PersistentMapping({'cols': "columns"}),
                PersistentMapping({'cols': 'columns'})]
                )})

        unwrapped_config = unwrap_persistence(config)

        self.assert_unwrapped_persistence(unwrapped_config)

    def test_unwrap_partial_persistance_with_list(self):
        config = PersistentMapping(
            {'default': list([
                PersistentMapping({'cols': "columns"}),
                PersistentMapping({'cols': 'columns'})])})

        unwrapped_config = unwrap_persistence(config)

        self.assert_unwrapped_persistence(unwrapped_config)

    def test_unwrap_partial_persistance_with_dict(self):
        config = dict(
            {'default': PersistentList([
                PersistentMapping({'cols': "columns"}),
                PersistentMapping({'cols': 'columns'})])})

        unwrapped_config = unwrap_persistence(config)

        self.assert_unwrapped_persistence(unwrapped_config)

    def test_unwrap_partial_persistance_with_tuple(self):
        config = PersistentMapping(
            {'default': tuple([
                PersistentMapping({'cols': "columns"}),
                PersistentMapping({'cols': 'columns'})])})

        unwrapped_config = unwrap_persistence(config)

        self.assert_unwrapped_persistence(unwrapped_config)

    def test_unwrap_partial_persistance_with_set(self):
        config = PersistentMapping(
            {'default': set(['value1', 'value2'])})

        unwrapped_config = unwrap_persistence(config)

        self.assert_unwrapped_persistence(unwrapped_config)



from Acquisition._Acquisition import aq_inner
from ftw.simplelayout import _
from ftw.simplelayout.browser.blocks.base import BaseBlock
from ftw.simplelayout.contenttypes.behaviors import ITeaser
from ftw.simplelayout.contenttypes.contents.textblock import ITextBlockSchema
from ftw.simplelayout.images.cropping.behaviors import ICroppedImageInOverlay
from ftw.simplelayout.images.cropping.behaviors import IImageCropping
from ftw.simplelayout.images.interfaces import IImageLimits
from ftw.simplelayout.interfaces import ISimplelayoutActions
from ftw.simplelayout.utils import normalize_portal_type
from plone.memoize.instance import memoize
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import queryMultiAdapter
from zope.i18n import translate


# Order is important
DISALLOWED_CSS_CLASSES =[
    'sl-layout-content',
    'sl-layout',
    'sl-block',
    'sl-columns',
    'sl-column',
    'sl-simplelayout',
]


class TextBlockView(BaseBlock):

    template = ViewPageTemplateFile('templates/textblock.pt')

    @property
    def teaser_url(self):
        teaser = ITeaser(self.context)
        if not teaser:
            return None

        if teaser.internal_link and teaser.internal_link.to_object:
            return teaser.internal_link.to_object.absolute_url()
        elif teaser.external_link:
            return teaser.external_link
        else:
            return None

    def get_image_data(self):
        """
        This method returns a dictionary containing all the data needed
        to render the image in the template.
        """
        if not self.context.image:
            return

        scale = self._get_image_scale()
        if not scale:
            return

        data = {
            'wrapper_css_classes': ' '.join(
                ['sl-image', self._get_image_scale_name(),
                 self._get_image_float()]
            ),
            'link_url': '',
            'link_title': self._get_link_title(),
            'link_css_classes': '',
            'image_tag': scale.tag(
                alt=self._get_image_alt_text(),
                title=None,
                height=None,
                width=None
            ),
        }

        if self.teaser_url:
            data.update({
                'link_url': self.teaser_url,
            })
            return data

        if self.context.open_image_in_overlay:
            # Get the scale defined in `ftw.colorbox`.
            force_original = False
            if ICroppedImageInOverlay.providedBy(self.context):
                force_original = not self.context.use_cropped_image_for_overlay
            else:
                force_original = True

            image_scale = self._get_image_scale('colorbox', force_original)
            # Don't fail if the scale has been removed in the policy.
            link_url = image_scale.url if image_scale else ''

            data.update({
                'link_url': link_url,
                'link_css_classes': 'colorboxLink',
            })
            return data

        return data

    def _get_link_title(self):
        alttext = ITextBlockSchema(self.context).image_alt_text

        if not alttext:
            return ''

        elif self.context.open_image_in_overlay and not self.teaser_url:
            return translate(_(u'image_link_alttext',
                               default=u'${title}, enlarged picture.',
                               mapping={'title': alttext}),
                             context=self.request)
        else:
            return alttext

    def _get_image_alt_text(self):
        if self.context.open_image_in_overlay or self.teaser_url:
            return ''
        else:
            return self._get_link_title()

    @memoize
    def _get_image_scale_name(self):
        scale = self.blockconfig.get('scale', '')
        if scale:
            return scale
        else:
            return self._get_default_scale()

    def _get_image_scale(self, scale=None, force_original=False):
        if not scale:
            scale = self._get_image_scale_name()
        scaler = self.context.restrictedTraverse('@@images')

        if not force_original and IImageCropping.providedBy(self.context) and \
                self.context.cropped_image:
            return scaler.scale('cropped_image', scale=scale)
        return scaler.scale('image', scale=scale)

    @memoize
    def _get_default_actions(self):
        normalized_portal_type = normalize_portal_type(
            self.context.portal_type)

        actions = queryMultiAdapter(
            (self.context, self.request),
            ISimplelayoutActions,
            name='{0}-actions'.format(normalized_portal_type))
        return actions.specific_actions().items()[0][1]

    @memoize
    def _get_default_scale(self):
        return self._get_default_actions()['data-scale']

    @memoize
    def _get_image_float(self):
        image_float = self.blockconfig.get('imagefloat', '')

        if image_float:
            return image_float
        else:
            return self._get_default_image_float()

    @memoize
    def _get_default_image_float(self):
        return self._get_default_actions()['data-imagefloat']

    @property
    def can_add(self):
        context = aq_inner(self.context)
        mtool = getToolByName(context, 'portal_membership')
        permission = mtool.checkPermission(
            'ftw.simplelayout: Add TextBlock', context)
        return bool(permission)

    def show_limit_indicator(self):
        return self.can_add

    def get_sl_safe_markup(self):
        # XXX: This should be implemented as a portal transform, in order to
        # use the right tool for it and get caching for free.
        text = self.context.text

        if text:
            safe_html = text.output
            for css_class in DISALLOWED_CSS_CLASSES:
                safe_html = safe_html.replace(css_class, '')
            return safe_html
        return ''



from ftw.logo.interfaces import IFtwLogo
from ftw.simplelayout.opengraph.og_site_root import PloneRootOpenGraph
from plone import api
from plone.app.caching.interfaces import IETagValue
from Products.CMFPlone.interfaces import IPloneSiteRoot
from zope.component import adapts
from zope.component import getMultiAdapter
from zope.interface import Interface


class FtwLogoRootOpenGraph(PloneRootOpenGraph):
    adapts(IPloneSiteRoot, IFtwLogo, Interface)

    def get_cache_key(self):
        adapter = getMultiAdapter((api.portal.get(), self.request),
                                  IETagValue,
                                  name='logo-viewlet')
        return adapter()

    def get_image_url(self):
        """ftw.logo image"""

        return '{}/@@logo/logo/get_logo?r={}'.format(
            api.portal.get().absolute_url(),
            self.get_cache_key()
        )



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING
from ftw.simplelayout.testing import SimplelayoutTestCase
from ftw.testbrowser import browsing


class TestToolBoxView(SimplelayoutTestCase):

    layer = FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.setup_sample_ftis(self.portal)
        self.setup_block_views()

        self.container = create(Builder('sample container'))

    @browsing
    def test_addable_blocks(self, browser):
        browser.login().visit(self.container, view='@@sl-toolbox-view')
        response = browser.json

        self.assertEquals(
            [
                {u'formUrl': u'http://nohost/plone/samplecontainer/++add_block++SampleBlock',
                 u'contentType': u'sampleblock',
                 u'description': u'',
                 u'actions': {u'edit': {u'href': u'http://nohost/plone/samplecontainer/sl-ajax-edit-block-view',
                                        u'class': u'edit sl-icon-edit',
                                        u'title': u'Edit block'},
                              u'move': {u'class': u'move sl-icon-move',
                                        u'title': u'Move block'},
                              u'delete': {u'href': u'http://nohost/plone/samplecontainer/sl-ajax-delete-blocks-view',
                                          u'class': u'delete sl-icon-delete',
                                          u'title': u'Delete block'}},
                 u'title': u'SampleBlock'}
            ],
            response['blocks'])

    @browsing
    def test_layout_actions(self, browser):
        self.maxDiff = None
        browser.login().visit(self.container, view='@@sl-toolbox-view')
        response = browser.json

        self.assertEquals(
            {u'move': {u'class': u'sl-icon-move move',
                       u'title': u'Move layout'},
             u'delete': {u'class': u'sl-icon-delete delete',
                         u'title': u'Delete layout'},
             u'layout-reverse': {u'class': u'sl-icon-layout-reverse reload',
                                 u'data-layoutreverse': u'layout-reverse',
                                 u'href': u'http://nohost/plone/samplecontainer/sl-ajax-reload-layout-view',
                                 u'rules': [2, 3],
                                 u'title': u'Invert layout'},
             u'layout112': {u'class': u'sl-icon-layout112 reload',
                            u'title': u'1-1-2 layout',
                            u'href': u'http://nohost/plone/samplecontainer/sl-ajax-reload-layout-view',
                            u'rules': [3],
                            u'data-layout112': u'layout112'},
             u'golden-ratio': {u'class': u'sl-icon-golden-ratio reload',
                               u'title': u'golden ratio',
                               u'href': u'http://nohost/plone/samplecontainer/sl-ajax-reload-layout-view',
                               u'data-golden_ratio': u'golden-ratio',
                               u'rules': [2]},
             u'layout121': {u'class': u'sl-icon-layout121 reload',
                            u'title': u'1-2-1 layout',
                            u'href': u'http://nohost/plone/samplecontainer/sl-ajax-reload-layout-view',
                            u'data-layout121': u'layout121',
                            u'rules': [3]},
             u'layout13': {u'class': u'sl-icon-layout13 reload',
                           u'data-layout13': u'layout13',
                           u'href': u'http://nohost/plone/samplecontainer/sl-ajax-reload-layout-view',
                           u'rules': [2],
                           u'title': u'1-3 layout'}},
            response['layoutActions'])
